@@ -10,6 +10,8 @@
 
 static void print_usage(FILE *stream);
 
+int globalverbose=0;
+
 int main(int argc, char *argv[])
 {
   optind = 1;
@@ -26,6 +28,10 @@ int main(int argc, char *argv[])
   }
 
   if(optind < argc){
+    if(!strcmp(argv[optind], "-verbose")){
+      optind++;
+      globalverbose=1;
+    }
     if(!strcmp(argv[optind], "help")){
       print_usage(stdout);
       exit(EXIT_SUCCESS);
@@ -82,6 +88,10 @@ int main(int argc, char *argv[])
 	    create_skeleton(argc-1, argv+1);
 	    exit(EXIT_FAILURE);
     }
+    else if(!strcmp(argv[optind], "addnodeids")){
+	    add_nodeids(argc-1, argv+1);
+	    exit(EXIT_FAILURE);
+    }
 
     else {
       fprintf(stderr, "Unknown command, %s.\n"
@@ -102,6 +112,7 @@ static void print_usage(FILE *stream){
 	  "  ccs_tool [options] <command>\n"
 	  "\n"
 	  "Options:\n"
+	  "  -verbose            Make some operations print more details.\n"
 	  "  -h                  Print this usage and exit.\n"
 	  "  -V                  Print version information and exit.\n"
 	  "\n"
@@ -116,5 +127,6 @@ static void print_usage(FILE *stream){
 	  "  addfence <fencedev> Add a new fence device\n"
 	  "  delfence <fencedev> Delete a fence device\n"
 	  "  create              Create a skeleton config file\n"
+	  "  addnodeids          Assign node ID numbers to all nodes\n"
 	  "\n");
 }
