@@ -1,7 +1,7 @@
 /******************************************************************************
 *******************************************************************************
 **
-**  Copyright (C) 2005 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2005-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -16,11 +16,16 @@
 #include <sys/stat.h>
 #include <getopt.h>
 #include <errno.h>
+#include <assert.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
 
 #include <libxml/tree.h>
 
 #include "update.h"
 
+#define MAX_NODES 256
 #define DEFAULT_CONFIG_FILE "/etc/cluster/cluster.conf"
 char *prog_name = "ccs_tool";
 
@@ -35,9 +40,9 @@ do { \
 struct option_info
 {
 	char *name;
+	char *altname;
 	char *votes;
 	char *nodeid;
-	char *mcast_if;
 	char *mcast_addr;
 	char *fence_type;
 	char *configfile;
@@ -54,8 +59,8 @@ static void config_usage(int rw)
 	{
 		fprintf(stderr, " -o --outputfile    Name of output file (defaults to same as --configfile)\n");
 		fprintf(stderr, " -C --no_ccs        Don't tell CCSD about this change\n");
-		fprintf(stderr, "                    (default: run \"ccs_tool update\" if file is updated in place)\n");
-		fprintf(stderr, " -F --force_ccs     Force \"ccs_tool update\" even if input & output files differ\n");
+		fprintf(stderr, "                    default: run \"ccs_tool update\" if file is updated in place)\n");
+		fprintf(stderr, " -F --force_ccs     Force \"ccs_tool upgrade\" even if input & output files differ\n");
 	}
 }
 
@@ -88,10 +93,10 @@ static void create_usage(const char *name)
 	  "eg:\n"
 	  "  ccs_tool create MyCluster\n"
 	  "  ccs_tool addfence apc fence_apc ipaddr=apc.domain.net user=apc password=apc\n"
-	  "  ccs_tool addnode node1 -v 1 -f apc port=1\n"
-	  "  ccs_tool addnode node2 -v 1 -f apc port=2\n"
-	  "  ccs_tool addnode node3 -v 1 -f apc port=3\n"
-	  "  ccs_tool addnode node4 -v 1 -f apc port=4\n"
+	  "  ccs_tool addnode node1 -n 1 -f apc port=1\n"
+	  "  ccs_tool addnode node2 -n 2 -f apc port=2\n"
+	  "  ccs_tool addnode node3 -n 3 -f apc port=3\n"
+	  "  ccs_tool addnode node4 -n 4 -f apc port=4\n"
           "\n");
 
 	exit(0);
@@ -128,14 +133,29 @@ static void delnode_usage(const char *name)
 	exit(0);
 }
 
+static void addnodeid_usage(const char *name)
+{
+	fprintf(stderr, "Add node IDs to all nodes in the config file that don't have them.\n");
+	fprintf(stderr, "Nodes with IDs will not be afftected, so you can run this as many times\n");
+	fprintf(stderr, "as you like without doing any harm.\n");
+	fprintf(stderr, "It will optionally add a multicast address to the cluster config too.\n");
+	fprintf(stderr, "\n");
+	fprintf(stderr, "Usage: %s %s [options] <name>\n", prog_name, name);
+	fprintf(stderr, " -n --nodeid        Nodeid to start with (default 1)\n");
+	fprintf(stderr, " -m --multicast     Set or change the multicast address\n");
+	fprintf(stderr, " -v --verbose       Print nodeids that are assigned\n");
+	config_usage(1);
+	help_usage();
+
+	exit(0);
+}
+
 static void addnode_usage(const char *name)
 {
 	fprintf(stderr, "Usage: %s %s [options] <nodename> [<fencearg>=<value>]...\n", prog_name, name);
-	fprintf(stderr, " -v --votes         Number of votes for this node\n");
-	fprintf(stderr, " -n --nodeid        Nodeid (optional)\n");
-	fprintf(stderr, " -i --interface     Interface name (needed if multicast is used)\n");
-	fprintf(stderr, " -m --multicast     Multicast address (only needed for first node in a cman\n");
-        fprintf(stderr, "                    multicast cluster)\n");
+	fprintf(stderr, " -n --nodeid        Nodeid (required)\n");
+	fprintf(stderr, " -v --votes         Number of votes for this node (default 1)\n");
+	fprintf(stderr, " -a --altname       Alternative name/interface for multihomed hosts\n");
 	fprintf(stderr, " -f --fence_type    Type of fencing to use\n");
 	config_usage(1);
 	help_usage();
@@ -144,14 +164,34 @@ static void addnode_usage(const char *name)
 	fprintf(stderr, "Examples:\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Add a new node to default configuration file:\n");
-	fprintf(stderr, "  %s %s -v 1 -f manual ipaddr=newnode\n", prog_name, name);
+	fprintf(stderr, "  %s %s -n 1 -f manual ipaddr=newnode\n", prog_name, name);
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Add a new node and dump config file to stdout rather than save it\n");
-	fprintf(stderr, "  %s %s -v 1 -f apc -i eth0 -o - newnode.temp.net port=1\n", prog_name, name);
+	fprintf(stderr, "  %s %s -n 2 -f apc -o- newnode.temp.net port=1\n", prog_name, name);
 
 	exit(0);
 }
 
+/* Is it really ?
+ * Actually, we don't check that this is a valid multicast address(!),
+ * merely that it is a valid IP[46] address.
+ */
+static int valid_mcast_addr(char *mcast)
+{
+        struct addrinfo *ainfo;
+        struct addrinfo ahints;
+	int ret;
+
+        memset(&ahints, 0, sizeof(ahints));
+
+        ret = getaddrinfo(mcast, NULL, &ahints, &ainfo);
+	if (ret) {
+		freeaddrinfo(ainfo);
+		return 0;
+	}
+	return 1;
+}
+
 static void save_file(xmlDoc *doc, struct option_info *ninfo)
 {
 	char tmpfile[strlen(ninfo->outputfile)+5];
@@ -345,42 +385,19 @@ static xmlNode *get_by_nodeid(xmlNode *root, int nodeid)
 }
 
 
-/* Get the multicast address (if present) from the first node.
- * If one has it, they all must.
- * Note that if there are no nodes in the config file then
- * we return the command-line multicast address (which may also be NULL!)
+/* Get the multicast address node.
  */
-static xmlChar *find_multicast_addr(xmlNode *clusternodes, struct option_info *ninfo)
+static xmlNode *find_multicast_addr(xmlNode *clusternodes)
 {
-	xmlChar *mc_addr = NULL;
-
-	xmlNode *clnode = findnode(clusternodes, "clusternode");
+	xmlNode *clnode = findnode(clusternodes, "cman");
 	if (clnode)
 	{
 		xmlNode *mcast = findnode(clnode, "multicast");
-		if (mcast)
-		{
-			mc_addr = xmlGetProp(mcast, BAD_CAST "addr");
-		}
+		return mcast;
 	}
-	else
-		mc_addr = (xmlChar *)ninfo->mcast_addr;
-	return mc_addr;
+	return NULL;
 }
 
-static xmlChar *get_mcast_if(xmlNode *clusternode)
-{
-	xmlChar *mc_if = NULL;
-
-	xmlNode *mcast = findnode(clusternode, "multicast");
-	if (mcast)
-	{
-		mc_if = xmlGetProp(mcast, BAD_CAST "interface");
-	}
-	return mc_if;
-}
-
-
 static xmlNode *find_node(xmlNode *clusternodes, char *nodename)
 {
 	xmlNode *cur_node;
@@ -400,7 +417,7 @@ static xmlNode *find_node(xmlNode *clusternodes, char *nodename)
 /* Print name=value pairs for a (n XML) node.
  * "ignore" is a string to ignore if present as a property (probably already printed on the main line)
  */
-static void print_properties(xmlNode *node, char *prefix, char *ignore)
+static void print_properties(xmlNode *node, char *prefix, char *ignore, char *ignore2)
 {
 	xmlAttr *attr;
 	int done_prefix = 0;
@@ -409,7 +426,9 @@ static void print_properties(xmlNode *node, char *prefix, char *ignore)
 	{
 		/* Don't print "name=" */
 		if (strcmp((char *)attr->name, "name") &&
-		    strcmp((char *)attr->name, ignore) )
+		    strcmp((char *)attr->name, ignore) &&
+		    strcmp((char *)attr->name, ignore2)
+			)
 		{
 			if (!done_prefix)
 			{
@@ -461,7 +480,6 @@ static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
 	xmlNode *newfence;
 	xmlNode *newfencemethod;
 	xmlNode *newfencedevice;
-	xmlChar *multicast_addr;
 
 	clusternodes = findnode(root_element, "clusternodes");
 	if (!clusternodes)
@@ -472,43 +490,23 @@ static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
 		die("node %s already exists in %s\n", ninfo->name, ninfo->configfile);
 
 	/* Check for duplicate node ID */
-	if (ninfo->nodeid && get_by_nodeid(root_element, atoi((char *)ninfo->nodeid)))
+	if (!ninfo->nodeid)
+		die("nodeid not specified\n");
+
+	if (get_by_nodeid(root_element, atoi((char *)ninfo->nodeid)))
 		die("nodeid %s already in use\n", ninfo->nodeid);
 
         /* Don't allow random fence types */
 	if (!valid_fence_type(root_element, ninfo->fence_type))
 		die("fence type '%s' not known\n", ninfo->fence_type);
 
-	/*
-	 * Check for a multicast line on the first node. If it's there then
-	 * the user must have supplied an interface on the command-line.
-	 */
-	multicast_addr = find_multicast_addr(clusternodes, ninfo);
-	if (multicast_addr && !ninfo->mcast_if)
-		die("no interface specified, but cluster.conf uses multicast\n");
-
-	/* We could ignore this, but I'd rather point out the user's mistake */
-	if (!multicast_addr && ninfo->mcast_if)
-		die("interface was specified, but cluster.conf is not set up for multicast\n");
-
 	/* Add the new node */
 	newnode = xmlNewNode(NULL, BAD_CAST "clusternode");
 	xmlSetProp(newnode, BAD_CAST "name", BAD_CAST ninfo->name);
 	xmlSetProp(newnode, BAD_CAST "votes", BAD_CAST ninfo->votes);
-	if (ninfo->nodeid)
-		xmlSetProp(newnode, BAD_CAST "nodeid", BAD_CAST ninfo->nodeid);
+	xmlSetProp(newnode, BAD_CAST "nodeid", BAD_CAST ninfo->nodeid);
 	xmlAddChild(clusternodes, newnode);
 
-	if (multicast_addr)
-	{
-		xmlNode *mcastnode;
-
-		mcastnode = xmlNewNode(NULL, BAD_CAST "multicast");
-		xmlSetProp(mcastnode, BAD_CAST "addr", multicast_addr);
-		xmlSetProp(mcastnode, BAD_CAST "interface", BAD_CAST ninfo->mcast_if);
-		xmlAddChild(newnode, mcastnode);
-	}
-
 	/* Add the fence attributes */
 	newfence = xmlNewNode(NULL, BAD_CAST "fence");
 	newfencemethod = xmlNewNode(NULL, BAD_CAST "method");
@@ -573,8 +571,7 @@ struct option addnode_options[] =
 {
       { "votes", required_argument, NULL, 'v'},
       { "nodeid", required_argument, NULL, 'n'},
-      { "interface", required_argument, NULL, 'i'},
-      { "multicast", required_argument, NULL, 'm'},
+      { "altname", required_argument, NULL, 'a'},
       { "fence_type", required_argument, NULL, 'f'},
       { "outputfile", required_argument, NULL, 'o'},
       { "configfile", required_argument, NULL, 'c'},
@@ -601,6 +598,16 @@ struct option addfence_options[] =
       { NULL, 0, NULL, 0 },
 };
 
+struct option addnodeid_options[] =
+{
+      { "outputfile", required_argument, NULL, 'o'},
+      { "configfile", required_argument, NULL, 'c'},
+      { "multicast", required_argument, NULL, 'm'},
+      { "nodeid", no_argument, NULL, 'n'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0 },
+};
+
 struct option list_options[] =
 {
       { "configfile", required_argument, NULL, 'c'},
@@ -609,6 +616,165 @@ struct option list_options[] =
 };
 
 
+static int next_nodeid(int startid, int *nodeids, int nodecount)
+{
+	int i;
+	int nextid = startid;
+
+retry:
+	for (i=0; i<nodecount; i++)
+	{
+		if (nodeids[i] == nextid)
+		{
+			nextid++;
+			goto retry;
+		}
+	}
+
+	return nextid;
+}
+
+void add_nodeids(int argc, char **argv)
+{
+	struct option_info ninfo;
+	unsigned char *nodenames[MAX_NODES];
+	xmlDoc *doc;
+	xmlNode *root_element;
+	xmlNode *clusternodes;
+	xmlNode *cur_node;
+	xmlNode *mcast;
+	int  verbose = 0;
+	int  opt;
+	int  i;
+	int  nodenumbers[MAX_NODES];
+	int  nodeidx;
+	int  totalnodes;
+	int  nextid;
+
+	memset(nodenames, 0, sizeof(nodenames));
+	memset(nodenumbers, 0, sizeof(nodenumbers));
+	memset(&ninfo, 0, sizeof(ninfo));
+	ninfo.nodeid = "1";
+
+	while ( (opt = getopt_long(argc, argv, "n:o:c:m:vh?", addnodeid_options, NULL)) != EOF)
+	{
+		switch(opt)
+		{
+		case 'n':
+			validate_int_arg(opt, optarg);
+			ninfo.nodeid = strdup(optarg);
+			break;
+
+		case 'c':
+			ninfo.configfile = strdup(optarg);
+			break;
+
+		case 'o':
+			ninfo.outputfile = strdup(optarg);
+			break;
+
+		case 'm':
+			if (!valid_mcast_addr(optarg)) {
+				fprintf(stderr, "%s is not a valid multicast address\n", optarg);
+				return;
+			}
+			ninfo.mcast_addr = strdup(optarg);
+			break;
+
+		case 'v':
+			verbose++;
+			break;
+
+		case '?':
+		default:
+			addnodeid_usage(argv[0]);
+		}
+	}
+
+	doc = open_configfile(&ninfo);
+
+	root_element = xmlDocGetRootElement(doc);
+
+	increment_version(root_element);
+
+	/* Warn if the cluster doesn't have a multicast address */
+	mcast = find_multicast_addr(root_element);
+	if (!mcast & !ninfo.mcast_addr) {
+		fprintf(stderr, "\nWARNING: The cluster does not have a multicast address.\n");
+		fprintf(stderr, "A default will be assigned a run-time which might not suit your installation\n\n");
+	}
+
+	if (ninfo.mcast_addr) {
+		if (!mcast) {
+			xmlNode *cman = xmlNewNode(NULL, BAD_CAST "cman");
+			mcast = xmlNewNode(NULL, BAD_CAST "multicast");
+
+			xmlAddChild(cman, mcast);
+			xmlAddChild(root_element, cman);
+		}
+		xmlSetProp(mcast, BAD_CAST "addr", BAD_CAST ninfo.mcast_addr);
+	}
+
+	/* Get a list of nodes that /do/ have nodeids so we don't generate
+	   any duplicates */
+	nodeidx=0;
+	clusternodes = findnode(root_element, "clusternodes");
+	if (!clusternodes)
+		die("Can't find \"clusternodes\" in %s\n", ninfo.configfile);
+
+
+	for (cur_node = clusternodes->children; cur_node; cur_node = cur_node->next)
+	{
+		if (cur_node->type == XML_ELEMENT_NODE && strcmp((char *)cur_node->name, "clusternode") == 0)
+		{
+			xmlChar *name   = xmlGetProp(cur_node, BAD_CAST "name");
+			xmlChar *nodeid = xmlGetProp(cur_node, BAD_CAST "nodeid");
+			nodenames[nodeidx]  = name;
+			if (nodeid)
+				nodenumbers[nodeidx] = atoi((char*)nodeid);
+			nodeidx++;
+		}
+	}
+	totalnodes = nodeidx;
+
+	/* Loop round nodes adding nodeIDs where they don't exist. */
+	nextid = next_nodeid(atoi(ninfo.nodeid), nodenumbers, totalnodes);
+	for (i=0; i<totalnodes; i++)
+	{
+		if (nodenumbers[i] == 0)
+		{
+			nodenumbers[i] = nextid;
+			nextid = next_nodeid(nextid, nodenumbers, totalnodes);
+			if (verbose)
+				fprintf(stderr, "Node %s now has id %d\n", nodenames[i], nodenumbers[i]);
+		}
+	}
+
+	/* Now write them into the tree */
+	nodeidx = 0;
+	for (cur_node = clusternodes->children; cur_node; cur_node = cur_node->next)
+	{
+		if (cur_node->type == XML_ELEMENT_NODE && strcmp((char *)cur_node->name, "clusternode") == 0)
+		{
+			char tmp[80];
+			xmlChar *name = xmlGetProp(cur_node, BAD_CAST "name");
+
+			assert(strcmp((char*)nodenames[nodeidx], (char*)name) == 0);
+
+			sprintf(tmp, "%d", nodenumbers[nodeidx]);
+			xmlSetProp(cur_node, BAD_CAST "nodeid", BAD_CAST tmp);
+			nodeidx++;
+		}
+	}
+
+
+	/* Write it out */
+	save_file(doc, &ninfo);
+
+	/* Shutdown libxml */
+	xmlCleanupParser();
+}
+
 void add_node(int argc, char **argv)
 {
 	struct option_info ninfo;
@@ -618,8 +784,9 @@ void add_node(int argc, char **argv)
 
 	memset(&ninfo, 0, sizeof(ninfo));
 	ninfo.tell_ccsd = 1;
+	ninfo.votes = "1";
 
-	while ( (opt = getopt_long(argc, argv, "v:n:i:m:f:o:c:CFh?", addnode_options, NULL)) != EOF)
+	while ( (opt = getopt_long(argc, argv, "v:n:a:f:o:c:CFh?", addnode_options, NULL)) != EOF)
 	{
 		switch(opt)
 		{
@@ -633,12 +800,8 @@ void add_node(int argc, char **argv)
 			ninfo.nodeid = optarg;
 			break;
 
-		case 'i':
-			ninfo.mcast_if = strdup(optarg);
-			break;
-
-		case 'm':
-			ninfo.mcast_addr = strdup(optarg);
+		case 'a':
+			ninfo.altname = strdup(optarg);
 			break;
 
 		case 'f':
@@ -673,7 +836,7 @@ void add_node(int argc, char **argv)
 	else
 		addnode_usage(argv[0]);
 
-	if (!ninfo.fence_type || !ninfo.votes)
+	if (!ninfo.fence_type)
 		addnode_usage(argv[0]);
 
 
@@ -753,7 +916,7 @@ void list_nodes(int argc, char **argv)
 	xmlNode *clusternodes;
 	xmlNode *fencenode = NULL;
 	xmlDocPtr doc;
-	xmlChar *mcast;
+	xmlNode *mcast;
 	struct option_info ninfo;
 	int opt;
 	int verbose = 0;
@@ -779,19 +942,20 @@ void list_nodes(int argc, char **argv)
 
 	root_element = xmlDocGetRootElement(doc);
 
-	clusternodes = findnode(root_element, "clusternodes");
-	if (!clusternodes)
-		die("Can't find \"clusternodes\" in %s\n", ninfo.configfile);
 
 	printf("\nCluster name: %s, config_version: %s\n\n",
 	       (char *)cluster_name(root_element),
 	       (char *)find_version(root_element));
 
-	mcast = find_multicast_addr(clusternodes, &ninfo);
-	if (mcast)
-		printf("Multicast address for cluster: %s\n\n", mcast);
+	clusternodes = findnode(root_element, "clusternodes");
+	if (!clusternodes)
+		die("Can't find \"clusternodes\" in %s\n", ninfo.configfile);
 
-	printf("Nodename                        Votes Nodeid Iface Fencetype\n");
+	mcast = find_multicast_addr(root_element);
+	if (mcast)
+		printf("Multicast address for cluster: %s\n\n", xmlGetProp(mcast, BAD_CAST "addr"));
+
+	printf("Nodename                        Votes Nodeid Fencetype\n");
 	for (cur_node = clusternodes->children; cur_node; cur_node = cur_node->next)
 	{
 		if (cur_node->type == XML_ELEMENT_NODE && strcmp((char *)cur_node->name, "clusternode") == 0)
@@ -800,16 +964,19 @@ void list_nodes(int argc, char **argv)
 			xmlChar *votes  = xmlGetProp(cur_node, BAD_CAST "votes");
 			xmlChar *nodeid = xmlGetProp(cur_node, BAD_CAST "nodeid");
 			xmlChar *ftype  = get_fence_type(cur_node, &fencenode);
-			xmlChar *mc_if  = get_mcast_if(cur_node);
 
-			printf("%-32s %3d    %-3s  %-5s %s\n", name, atoi((char *)votes),
-			       nodeid?nodeid:(xmlChar *)"",
-			       mc_if?mc_if:(xmlChar *)"",
+			if (!nodeid)
+				nodeid=(unsigned char *)"0";
+			if (!votes)
+				votes = (unsigned char *)"1";
+
+			printf("%-32s %3d  %3d    %s\n", name, atoi((char *)votes),
+			       atoi((char *)nodeid),
 			       ftype?ftype:(xmlChar *)"");
 			if (verbose)
 			{
-				print_properties(cur_node, "  Node properties: ", "votes");
-				print_properties(fencenode, "  Fence properties: ", "agent");
+				print_properties(cur_node, "  Node properties: ", "votes", "nodeid");
+				print_properties(fencenode, "  Fence properties: ", "agent", "");
 			}
 
 		}
@@ -1077,7 +1244,7 @@ void list_fences(int argc, char **argv)
 
 			printf("%-16s %s\n", name, agent);
 			if (verbose)
-				print_properties(cur_node, "  Properties: ", "agent");
+				print_properties(cur_node, "  Properties: ", "agent", "");
 		}
 	}
 }
