@@ -28,6 +28,7 @@
 #include <netdb.h>
 
 /* openais headers */
+#include <openais/service/swab.h>
 #include <openais/totem/totemip.h>
 #include <openais/totem/totempg.h>
 #include <openais/totem/aispoll.h>
@@ -134,7 +135,7 @@ static inline void objdb_get_int(struct objdb_iface_ver0 *objdb, unsigned int ob
  * Exports the interface for the service
  */
 static struct openais_service_handler cman_service_handler = {
-	.name		    		= (unsigned char *)"openais CMAN membership service 2.01",
+	.name		    		= (char *)"openais CMAN membership service 2.01",
 	.id			        = CMAN_SERVICE,
 	.flow_control			= OPENAIS_FLOW_CONTROL_NOT_REQUIRED,
 	.lib_exit_fn		       	= cman_exit_fn,
@@ -515,9 +516,11 @@ static int comms_init_ais(struct objdb_iface_ver0 *objdb)
 							"none", strlen("none")+1);
 		}
 
-		sprintf(tmp, "%d", 1);
-		objdb->object_key_create(object_handle, "secauth", strlen("secauth"),
-					 tmp, strlen(tmp)+1);
+		if (objdb_get_string(objdb, object_handle, "secauth", &value)) {
+			sprintf(tmp, "%d", 1);
+			objdb->object_key_create(object_handle, "secauth", strlen("secauth"),
+						 tmp, strlen(tmp)+1);
+		}
 
 		if (key_filename)
 		{
