@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -19,8 +19,8 @@
 #include <linux/completion.h>
 #include <linux/buffer_head.h>
 #include <asm/uaccess.h>
-#include <linux/gfs_ioctl.h>
 
+#include "gfs_ioctl.h"
 #include "gfs.h"
 #include "bmap.h"
 #include "dio.h"
@@ -434,8 +434,8 @@ gi_get_tune(struct gfs_inode *ip,
 	    unsigned int size,
 	    unsigned int *count)
 {
-        struct gfs_tune *gt = &ip->i_sbd->sd_tune;
-        int error = -ENOBUFS;
+	struct gfs_tune *gt = &ip->i_sbd->sd_tune;
+	int error = -ENOBUFS;
 
 	if (gi->gi_argc != 1)
 		return -EINVAL;
@@ -458,6 +458,7 @@ gi_get_tune(struct gfs_inode *ip,
         gfs_printf("logd_secs %u\n", gt->gt_logd_secs);
         gfs_printf("quotad_secs %u\n", gt->gt_quotad_secs);
         gfs_printf("inoded_secs %u\n", gt->gt_inoded_secs);
+        gfs_printf("glock_purge %u\n", gt->gt_glock_purge);
         gfs_printf("quota_simul_sync %u\n", gt->gt_quota_simul_sync);
         gfs_printf("quota_warn_period %u\n", gt->gt_quota_warn_period);
         gfs_printf("atime_quantum %u\n", gt->gt_atime_quantum);
@@ -482,6 +483,7 @@ gi_get_tune(struct gfs_inode *ip,
         gfs_printf("greedy_quantum %u\n", gt->gt_greedy_quantum);
         gfs_printf("greedy_max %u\n", gt->gt_greedy_max);
         gfs_printf("rgrp_try_threshold %u\n", gt->gt_rgrp_try_threshold);
+        gfs_printf("statfs_fast %u\n", gt->gt_statfs_fast);
 
         error = 0;
 
@@ -512,6 +514,7 @@ gi_set_tune(struct gfs_sbd *sdp, struct gfs_ioctl *gi)
 	struct gfs_tune *gt = &sdp->sd_tune;
  	char param[ARG_SIZE], value[ARG_SIZE];
 	unsigned int x;
+	int error;
 
 	if (!capable(CAP_SYS_ADMIN))
                 return -EACCES;
@@ -606,6 +609,11 @@ gi_set_tune(struct gfs_sbd *sdp, struct gfs_ioctl *gi)
 		tune_set(gt_inoded_secs, x);
 		wake_up_process(sdp->sd_inoded_process);
 
+	} else if (strcmp(param, "glock_purge") == 0) {
+		if (sscanf(value, "%u", &x) != 1)
+			return -EINVAL;
+		tune_set(gt_glock_purge, x);
+
 	} else if (strcmp(param, "quota_simul_sync") == 0) {
 		if (sscanf(value, "%u", &x) != 1 || !x)
 			return -EINVAL;
@@ -746,6 +754,16 @@ gi_set_tune(struct gfs_sbd *sdp, struct gfs_ioctl *gi)
 			return -EINVAL;
 		tune_set(gt_rgrp_try_threshold, x);
 
+	} else if (strcmp(param, "statfs_fast") == 0) {
+		if (sscanf(value, "%u", &x) != 1)
+			return -EINVAL;
+		error = gfs_statfs_init(sdp, x);
+		if (error)
+			return error;
+		else
+			tune_set(gt_statfs_fast, x);
+
+
 	} else
 		return -EINVAL;
 
@@ -1231,7 +1249,7 @@ gi_do_hfile_write(struct gfs_sbd *sdp, struct gfs_ioctl *gi)
 	}
 
 	error = gfs_writei(ip, gi->gi_data, gi->gi_offset, gi->gi_size,
-					   gfs_copy_from_user, NULL);
+			   gfs_copy_from_user, NULL);
 
 	gfs_trans_end(sdp);
 
@@ -1464,6 +1482,8 @@ gfs_ioctl_i(struct gfs_inode *ip, void *arg)
 		error = gi_set_file_flag(ip, &gi);
 	else if (strcmp(arg0, "get_file_meta") == 0)
 		error = gi_get_file_meta(ip, &gi);
+	else if (strcmp(arg0, "get_file_meta_quota") == 0)
+		error = gi_get_file_meta(ip->i_sbd->sd_qinode, &gi);
 	else if (strcmp(arg0, "do_file_flush") == 0)
 		error = gi_do_file_flush(ip, &gi);
 	else if (strcmp(arg0, "get_hfile_stat") == 0)
