@@ -63,7 +63,7 @@ void version(void)
 
 int read_cmdline(int argc, char **argv, struct options *opts)
 {
-	char c;
+	int c;
 
 	while((c = getopt(argc, argv, "hnqvyV")) != -1) {
 		switch(c) {
@@ -276,8 +276,6 @@ int main(int argc, char **argv)
 		}
 		else
 			log_notice("Pass5 complete      \n");
-		/*	print_map(sbp->bl, sbp->last_fs_block); */
-
 		log_notice("Writing changes to disk\n");
 	}
 	destroy(sbp);
