@@ -3,7 +3,7 @@
 **
 **  gfs2_convert - convert a gfs1 filesystem into a gfs2 filesystem.
 **
-**  Copyright (C) 2006 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2006-2007 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -773,17 +773,6 @@ static int init(struct gfs2_sbd *sbp)
 		close(sbp->device_fd);
 		exit(-1);
 	}
-	if (sbp->sd_sb.sb_bsize != GFS2_DEFAULT_BSIZE) {
-		log_crit("Error: GFS2 only supports a 4K block size.\n");
-		log_crit("Volume %s has a block size of ", device);
-		if (!(sbp->sd_sb.sb_bsize % 1024)) /* if even K boundary */
-			log_crit("%dK bytes.\n", sbp->sd_sb.sb_bsize / 1024);
-		else
-			log_crit("%d bytes.\n", sbp->sd_sb.sb_bsize);
-		log_crit("This file system cannot be converted with this tool.\n");
-		close(sbp->device_fd);
-		exit(-1);
-	}
 	/* get gfs1 rindex inode - gfs1's rindex inode ptr became __pad2 */
 	gfs2_inum_in(&inum, (char *)&raw_gfs1_ondisk_sb.sb_rindex_di);
 	sbp->md.riinode = gfs2_load_inode(sbp, inum.no_addr);
@@ -801,7 +790,7 @@ static int init(struct gfs2_sbd *sbp)
 	/* look like a directory, temporarily.                               */
 	sbp->md.riinode->i_di.di_mode &= ~S_IFMT;
 	sbp->md.riinode->i_di.di_mode |= S_IFDIR; 
-	if (ri_update(sbp, &rgcount)){
+	if (ri_update(sbp, 0, &rgcount)){
 		log_crit("Unable to fill in resource group information.\n");
 		return -1;
 	}
@@ -1181,7 +1170,7 @@ int main(int argc, char **argv)
 		/* Now delete the now-obsolete gfs1 files: */
 		remove_obsolete_gfs1(&sb2);
 		/* Now free all the in memory */
-		gfs2_rgrp_free(&sb2, updated);
+		gfs2_rgrp_free(&sb2.rglist, updated);
 		log_notice("Committing changes to disk.\n");
 		fflush(stdout);
 		/* Set filesystem type in superblock to gfs2.  We do this at the */
