@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -23,6 +23,7 @@
 #include <time.h>
 #include <errno.h>
 #include <stdarg.h>
+#include <mntent.h>
 
 #include <linux/types.h>
 #include "libgfs2.h"
@@ -58,6 +59,7 @@ print_usage(void)
 	printf("\n");
 	printf("Options:\n");
 	printf("\n");
+	printf("  -b <bytes>       Filesystem block size\n");
 	printf("  -c <MB>          Size of quota change file\n");
 	printf("  -D               Enable debugging code\n");
 	printf("  -h               Print this help, then exit\n");
@@ -86,14 +88,17 @@ decode_arguments(int argc, char *argv[], struct gfs2_sbd *sdp)
 	int cont = TRUE;
 	int optchar;
 
-	sdp->device_name = NULL;
+	memset(sdp->device_name, 0, sizeof(sdp->device_name));
 	sdp->md.journals = 1;
 	strcpy(sdp->lockproto, "lock_nolock");
 
 	while (cont) {
-		optchar = getopt(argc, argv, "-c:DhJ:j:Op:qr:t:u:VX");
+		optchar = getopt(argc, argv, "-b:c:DhJ:j:Op:qr:t:u:VX");
 
 		switch (optchar) {
+		case 'b':
+			sdp->bsize = atoi(optarg);
+			break;
 
 		case 'c':
 			sdp->qcsize = atoi(optarg);
@@ -169,10 +174,10 @@ decode_arguments(int argc, char *argv[], struct gfs2_sbd *sdp)
 		case 1:
 			if (strcmp(optarg, "gfs2") == 0)
 				continue;
-			if (sdp->device_name) {
+			if (sdp->device_name[0]) {
 				die("More than one device specified (try -h for help)");
 			} 
-			sdp->device_name = optarg;
+			strcpy(sdp->device_name, optarg);
 			break;
 
 		default:
@@ -181,11 +186,10 @@ decode_arguments(int argc, char *argv[], struct gfs2_sbd *sdp)
 		};
 	}
 
-	if ((sdp->device_name == NULL) && (optind < argc)) {
-		sdp->device_name = argv[optind++];
-	}
+	if ((sdp->device_name[0] == 0) && (optind < argc))
+		strcpy(sdp->device_name, argv[optind++]);
 
-	if (sdp->device_name == NULL)
+	if (sdp->device_name[0] == '\0')
 		die("no device specified (try -h for help)\n");
 
 	if (optind < argc)
@@ -193,6 +197,7 @@ decode_arguments(int argc, char *argv[], struct gfs2_sbd *sdp)
 
 	if (sdp->debug) {
 		printf("Command Line Arguments:\n");
+		printf("  bsize = %u\n", sdp->bsize);
 		printf("  qcsize = %u\n", sdp->qcsize);
 		printf("  jsize = %u\n", sdp->jsize);
 		printf("  journals = %u\n", sdp->md.journals);
@@ -264,8 +269,7 @@ static void are_you_sure(struct gfs2_sbd *sdp)
 		die("error identifying the contents of %s: %s\n",
 		    sdp->device_name, strerror(errno));
 
-	printf("This will destroy any data on %s.\n",
-	       sdp->device_name);
+	printf("This will destroy any data on %s.\n", sdp->device_name);
 	if (volume_id_probe_all(vid, 0, sdp->device_size) == 0)
 		printf("  It appears to contain a %s %s.\n", vid->type,
 			   vid->usage_id == VOLUME_ID_OTHER? "partition" : vid->usage);
@@ -279,6 +283,36 @@ static void are_you_sure(struct gfs2_sbd *sdp)
 		printf("\n");
 }
 
+/**
+ * check_mount - check to see if device is mounted
+ * @device: the device to create the filesystem on
+ *
+ */
+
+void check_mount(char *device)
+{
+	struct mntent *mnt;
+	FILE *fp;
+
+	if ((fp = setmntent("/proc/mounts", "r")) == NULL) {
+		die("error opening /proc/mounts");
+	}
+
+	while ((mnt = getmntent(fp)) != NULL) {
+		if (strcmp(device, mnt->mnt_fsname) == 0) {
+			printf("cannot create filesystem: ");
+			printf("%s appears to be mounted\n", device);
+			endmntent(fp);
+			exit(EXIT_FAILURE);
+			break;
+		}
+	}
+
+	endmntent(fp);
+
+	return;
+}
+
 /**
  * print_results - print out summary information
  * @sdp: the command line
@@ -354,6 +388,8 @@ main_mkfs(int argc, char *argv[])
 
 	verify_arguments(sdp);
 
+	check_mount(sdp->device_name);
+
 	sdp->device_fd = open(sdp->device_name, O_RDWR);
 	if (sdp->device_fd < 0)
 		die("can't open device %s: %s\n",
@@ -375,7 +411,7 @@ main_mkfs(int argc, char *argv[])
 
 	/* Build ondisk structures */
 
-	build_rgrps(sdp);
+	build_rgrps(sdp, TRUE);
 	build_root(sdp);
 	build_master(sdp);
 	build_sb(sdp);
