@@ -58,6 +58,7 @@ static struct opt_map opt_map[] = {
   { "nouser",   0, 1, 0  },             /* Forbid ordinary user to mount */
   { "owner",    0, 0, 0  },             /* Let the owner of the device mount */
   { "noowner",  0, 1, 0  },             /* Device owner has no special privs */
+  { "_netdev",  0, 0, 0  },             /* Network device required (netfs) */
   { NULL,       0, 0, 0	 }
 };
 
@@ -291,7 +292,6 @@ int get_sb(char *device, struct gen_sb *sb_out)
 
 		if (sb.sb_header.mh_magic != GFS_MAGIC ||
 		    sb.sb_header.mh_type != GFS_METATYPE_SB) {
-			gfs_sb_print(&sb);
 			die("there isn't a GFS filesystem on %s\n", device);
 		}
 
@@ -327,8 +327,11 @@ static int gfs_controld_connect(void)
 	int rv, fd;
 
 	fd = socket(PF_UNIX, SOCK_STREAM, 0);
-	if (fd < 0)
+	if (fd < 0) {
+		warn("can't create socket for gfs_controld connection: %s",
+		     strerror(errno));
 		goto out;
+	}
 
 	memset(&sun, 0, sizeof(sun));
 	sun.sun_family = AF_UNIX;
@@ -337,6 +340,7 @@ static int gfs_controld_connect(void)
 
 	rv = connect(fd, (struct sockaddr *) &sun, addrlen);
 	if (rv < 0) {
+		warn("can't connect to gfs_controld: %s", strerror(errno));
 		close(fd);
 		fd = rv;
 	}
@@ -406,10 +410,8 @@ int lock_dlm_join(struct mount_options *mo, struct gen_sb *sb)
 	i = 0;
 	do {
 		fd = gfs_controld_connect();
-		if (fd <= 0) {
-			warn("waiting for gfs_controld to start");
+		if (fd <= 0)
 			sleep(1);
-		}
 	} while (fd <= 0 && ++i < 10);
 
 	/* FIXME: should we start the daemon here? */
@@ -437,7 +439,7 @@ int lock_dlm_join(struct mount_options *mo, struct gen_sb *sb)
 	rv = snprintf(buf, MAXLINE, "join %s %s %s %s %s %s",
 		      dir, fsname, proto, table, options, mo->dev);
 	if (rv >= MAXLINE) {
-		warn("lock_dlm_join: message too long: %d \"%s\"", rv, buf);
+		warn("gfs_controld message too long: %d \"%s\"", rv, buf);
 		rv = -1;
 		goto out;
 	}
@@ -447,7 +449,7 @@ int lock_dlm_join(struct mount_options *mo, struct gen_sb *sb)
 
 	rv = write(fd, buf, sizeof(buf));
 	if (rv < 0) {
-		warn("lock_dlm_join: gfs_controld write error: %d", rv);
+		warn("gfs_controld write error: %s", strerror(errno));
 		goto out;
 	}
 
@@ -463,23 +465,74 @@ int lock_dlm_join(struct mount_options *mo, struct gen_sb *sb)
 	memset(buf, 0, sizeof(buf));
 	rv = read(fd, buf, sizeof(buf));
 	if (rv < 0) {
-		warn("lock_dlm_join: gfs_controld read 1 error: %d", rv);
+		warn("error reading result from gfs_controld: %s",
+		     strerror(errno));
 		goto out;
 	}
 	rv = atoi(buf);
 
-	if (rv == -EALREADY) {
+	switch (rv) {
+	case 0:
+		break;
+
+	case -EEXIST:
+		warn("mount group already exists. "
+		     "Duplicate locktable name %s, or %s already mounted",
+		     table, mo->dev);
+		goto out;
+
+	case -EPROTONOSUPPORT:
+		warn("lockproto not supported");
+		goto out;
+
+	case -EOPNOTSUPP:
+		warn("jid, first and id are reserved options");
+		goto out;
+
+	case -EBADFD:
+		warn("no colon found in table name");
+		goto out;
+
+	case -ENAMETOOLONG:
+		warn("fs name too long");
+		goto out;
+
+	case -EADDRINUSE:
+		warn("different fs appears to exist with the same name");
+		goto out;
+
+	case -EBUSY:
+		warn("mount point already used or other mount in progress");
+		goto out;
+
+	case -EALREADY:
 		log_debug("fs already mounted, adding mountpoint");
 		adding_another_mountpoint = 1;
 		rv = 0;
 		goto out;
-	}
-	if (rv < 0) {
-		warn("lock_dlm_join: gfs_controld join error: %d", rv);
-		if (rv == -EEXIST)
-			warn("lock_dlm_join: mountgroup already exists. "
-			     "Duplicate locktable name %s, or %s already "
-			     "mounted\n", table, mo->dev);
+
+	case -ENOMEM:
+		warn("out of memory");
+		goto out;
+
+	case -EBADR:
+		warn("fs is for a different cluster");
+		goto out;
+
+	case -ENOANO:
+		warn("node not a member of the default fence domain");
+		goto out;
+
+	case -EROFS:
+		warn("read-only mount invalid with spectator option");
+		goto out;
+
+	case -EMLINK:
+		warn("option string too long");
+		goto out;
+
+	default:
+		warn("gfs_controld join error: %d", rv);
 		goto out;
 	}
 
@@ -587,10 +640,8 @@ int lock_dlm_leave(struct mount_options *mo, struct gen_sb *sb, int mnterr)
 	i = 0;
 	do {
 		fd = gfs_controld_connect();
-		if (fd <= 0) {
-			warn("waiting for gfs_controld to start");
+		if (fd <= 0)
 			sleep(1);
-		}
 	} while (fd <= 0 && ++i < 10);
 
 	if (fd <= 0) {
@@ -667,10 +718,8 @@ int lock_dlm_remount(struct mount_options *mo, struct gen_sb *sb)
 	i = 0;
 	do {
 		fd = gfs_controld_connect();
-		if (fd <= 0) {
-			warn("waiting for gfs_controld to start");
+		if (fd <= 0)
 			sleep(1);
-		}
 	} while (fd <= 0 && ++i < 10);
 
 	if (fd <= 0) {
