@@ -23,8 +23,7 @@
 #include <sys/un.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
-#include <sysfs/dlist.h>
-#include <sysfs/libsysfs.h>
+#include <dirent.h>
 #include <ctype.h>
 #define _GNU_SOURCE
 #include <getopt.h>
@@ -41,7 +40,6 @@
 #define TIMEOUT_DEFAULT 60
 
 #define MAN_MSG "Please see man page for details.\n"
-#define DEFAULT_GETUID "/sbin/scsi_id -g -u -s /block/%n"
 
 int start_gnbd_clusterd(void)
 {
@@ -314,7 +312,8 @@ int list(void){
       printf("   timeout : %u\n", info->timeout);
     else
       printf("   timeout : no\n");
-    printf("       uid : %s\n", info->uid);
+    if (info->uid[0] != '\0')
+      printf("       uid : %s\n", info->uid);
     printf("\n");
     info++;
   }
@@ -342,54 +341,113 @@ void get_dev(char *path, int *major, int *minor)
   *minor = minor(stat_buf.st_rdev);
 }
 
+char *get_sysfs_info(char *path) {
+  int fd;
+  int bytes;
+  int count = 0;
 
-char *get_sysfs_name(char *dev_t){
-  unsigned char path[SYSFS_PATH_MAX], *name;
-  struct sysfs_directory *dir, *devdir;
-  struct dlist *devlist;
-
-  name = NULL;
-  if (sysfs_get_mnt_path(path, SYSFS_PATH_MAX) < 0){
-    printe("cannot get sysfs mount path for get_uid command : %s\n",
-           strerror(errno));
+  if ((fd = open(path, O_RDONLY)) < 0) {
+    printe("cannot open %s : %s\n", path, strerror(errno));
     exit(1);
   }
-  strcat(path, "/block");
-  dir = sysfs_open_directory(path);
+  while (count < 4096) {
+    bytes = read(fd, &sysfs_buf[count], 4096 - count);
+    if (bytes < 0 && errno != EINTR) {
+      printe("cannot read from %s : %s\n", path, strerror(errno));
+      exit(1);
+    }
+    if (bytes == 0)
+      break;
+    count += bytes;
+  }
+  if (sysfs_buf[count - 1] == '\n' || count == 4096)
+    sysfs_buf[count - 1] = '\0';
+  else
+    sysfs_buf[count] = '\0';
+  close(fd);
+  return sysfs_buf;
+}
+    
+#define SYSFS_PATH_MAX 64
+#define SYSFS_PATH_BASE "/sys/block"
+#define SYSFS_PATH_BASE_SIZE 10
+int get_sysfs_majmin(char *dev, int *major, int *minor)
+{
+  char path[SYSFS_PATH_MAX];
+  char *buf;
+
+  if (snprintf(path, SYSFS_PATH_MAX, "%s/%s/dev", SYSFS_PATH_BASE, dev) >=
+      SYSFS_PATH_MAX) {
+    printe("sysfs path name '%s/%s/dev' too long\n", SYSFS_PATH_BASE, dev);
+    exit(1);
+  }
+  buf = get_sysfs_info(path);
+  if (sscanf(buf, "%u:%u", major, minor) != 2){
+    printe("cannot parse %s entry '%s'\n", path, buf);
+    exit(1);
+  }
+  return 0;
+}
+
+int get_sysfs_range(char *dev, int *range)
+{
+  char path[SYSFS_PATH_MAX];
+  char *buf;
+
+  if (snprintf(path, SYSFS_PATH_MAX, "%s/%s/range", SYSFS_PATH_BASE, dev) >=
+      SYSFS_PATH_MAX) {
+    printe("sysfs path name '%s/%s/range' too long\n", SYSFS_PATH_BASE, dev);
+    exit(1);
+  }
+  buf = get_sysfs_info(path);
+  if (sscanf(buf, "%u", range) != 1){
+    printe("cannot parse %s etnry '%s'\n", path, buf);
+    exit(1);
+  }
+  return 0;
+}
+
+char *get_sysfs_name(int major, int minor){
+  char *name = NULL;
+  DIR *dir;
+  struct dirent *dp;
+
+  dir = opendir(SYSFS_PATH_BASE);
   if (!dir) {
-    printe("cannot open sysfs directory '%s' for get_uid command : %s\n",
-           path, strerror(errno));
+    printe("cannot open %s to find the device name for %d:%d : %s\n",
+           SYSFS_PATH_BASE, major, minor, strerror(errno));
     exit(1);
   }
-  devlist = sysfs_get_dir_subdirs(dir);
-  if (!devlist){
-    printe("cannot read sysfs subdirs for get_uid command : %s\n",
-           strerror(errno));
-    exit(1);
-  }
-  dlist_for_each_data(devlist, devdir, struct sysfs_directory) {
-    struct sysfs_attribute *attr;
-    attr = sysfs_get_directory_attribute(devdir, "dev");
-    if (!attr){
-      printe("cannot get 'dev' sysfs attribute for get_uid command : %s\n",
-             strerror(errno));
-      exit(1);
-    }
-    if (sysfs_read_attribute(attr) < 0){
-      printe("cannot get sysfs attribute data for get_uid command : %s\n",
-             strerror(errno));
-      exit(1);
-    }
-    if (strcmp(dev_t, attr->value) == 0){
-      name = strdup(devdir->name);
+  while ((dp = readdir(dir))) {
+    int dev_major, dev_minor, dev_range;
+    if (!strcmp(dp->d_name, ".") || !strcmp(dp->d_name, ".."))
+      continue;
+    get_sysfs_majmin(dp->d_name, &dev_major, &dev_minor);
+    get_sysfs_range(dp->d_name, &dev_range);
+    if (major == dev_major && minor >= dev_minor &&
+        minor < dev_minor + dev_range){
+      if (minor == dev_minor)
+        name = strdup(dp->d_name);
+      else {
+        name = malloc(SYSFS_PATH_MAX);
+        if (!name){
+          printe("cannot allocate memory for sysfs name : %s\n",
+                 strerror(errno));
+          exit(1);
+        }
+        sprintf(name, "%s/%s%d", dp->d_name, dp->d_name, minor - dev_minor);
+      }
       break;
     }
   }
-  if (!name){
-    printe("cannot find a sysfs block device for get_uid command\n");
+  if (closedir(dir) < 0){
+    printe("cannot close dir %s : %s\n", SYSFS_PATH_BASE, strerror(errno));
+    exit(1);
+  }
+  if (!name) {
+    printe("cannot find sysfs block device %d:%d\n", major, minor);
     exit(1);
   }
-  sysfs_close_directory(dir);
   return name;
 }
 
@@ -559,8 +617,7 @@ char *get_uid(char *format, char *path)
         if (!name){
           if (major == -1)
             get_dev(path, &major, &minor);
-          sprintf(temp, "%d:%d\n", major, minor);
-          name = get_sysfs_name(temp);
+          name = get_sysfs_name(major, minor);
         }
         len = strlen(name);
         if (len > SPACE_LEFT){
@@ -617,7 +674,7 @@ int usage(void){
 "  -u <uid>         manually set the Unique ID of a device (used with -e)\n"
 "  -U[command]      command to get the Unique ID of a device (used with -e)\n"
 "                   If no command is specificed, the default is\n"
-"                   \"/sbin/scsi_id -g -u -s /block/%%n\"\n"
+"                   \""DEFAULT_GETUID" %%n\"\n"
 "  -v               verbose output (useful with -l)\n"
 "  -V               version information\n");
   return 0;
@@ -744,6 +801,10 @@ int main(int argc, char **argv){
       return 1;
     }
   }
+  if (cached && (uid || uid_program)){
+    printe("the -c option may not be used with the -u or -U option\n" MAN_MSG);
+    return 1;
+  }
   if (timeout && cached){
     printe("the -t option may not be used with the -c option\n" MAN_MSG);
     return 1;
