@@ -116,9 +116,11 @@
 #include <unistd.h>
 #include <sys/wait.h>
 
+#ifndef __ia64__
 #ifndef DEBUG
 #define DEBUG			/* Record program counter of malloc/calloc */
 #endif				/* or realloc call; print misc stuff out */
+#endif
 
 /* Tunable stuff XXX This should be external */
 #define PARANOID		/* Trade off a bit of space and speed for
@@ -128,9 +130,10 @@
 				   for instance) */
 #undef  AGGR_RECLAIM		/* consolidate_all on free (*slow*) */
 
-#undef  STACKSIZE	/*4	   backtrace to store if DEBUG is set */
+//#undef  STACKSIZE	/*4	   backtrace to store if DEBUG is set */
+#define STACKSIZE 1		/* at least 1 gets you free addr */
 
-#undef  GDB_HOOK		/* Dump program addresses in malloc_table
+#undef	GDB_HOOK		/* Dump program addresses in malloc_table
 				   using a fork/exec of gdb (SLOW but fun)
 				   building this defeats the purpose of
 				   a bounded memory allocator, and is only
@@ -666,7 +669,7 @@ search_freestore(size_t size)
 
 #define stack_pointer(n) \
 	(__builtin_frame_address(n)?__builtin_return_address(n):NULL)
-			
+
 #define assign_address(_ptr, _cnt) \
 { \
 	switch(_cnt) { \
@@ -805,6 +808,12 @@ void
 free(void *p)
 {
 	memblock_t *b;
+#ifdef DEBUG
+#ifdef STACKSIZE
+	void *pc = __builtin_return_address(0);
+	int x;
+#endif
+#endif
 
 	if (!p) {
 #if 0
@@ -825,6 +834,28 @@ free(void *p)
 		die_or_return();
 	}
 
+	if (!is_valid_alloc(b)) {
+#ifdef DEBUG
+		if (!is_valid_free(b))
+			fprintf(stderr,
+				"free(%p) @ %p - Invalid address\n",
+				p, __builtin_return_address(0));
+		else
+#ifdef STACKSIZE
+			fprintf(stderr,
+				"free(%p) @ %p - Already free @ %p\n",
+				p, __builtin_return_address(0), b->mb_pc[0]);
+#else
+			fprintf(stderr,
+				"free(%p) @ %p - Already free\n",
+				p, __builtin_return_address(0));
+#endif
+
+#endif
+		pthread_mutex_unlock(&_alloc_mutex);
+		die_or_return();
+	}
+
 #ifdef PARANOID
 	/* Remove from the allocated list if we're tracking it. */
 	if (!remove_alloc_block(b)) {
@@ -835,20 +866,13 @@ free(void *p)
 	}
 #endif
 
-	if (!is_valid_alloc(b)) {
 #ifdef DEBUG
-		if (!is_valid_free(b))
-			fprintf(stderr,
-				"free(%p) @ %p - Invalid address\n",
-				p, __builtin_return_address(0));
-		else
-			fprintf(stderr,
-				"free(%p) @ %p - Already free\n",
-				p, __builtin_return_address(0));
+#ifdef STACKSIZE
+	for (x = 0; x < STACKSIZE; x++)
+		b->mb_pc[x] = NULL;
+	b->mb_pc[0] = pc;
+#endif
 #endif
-		pthread_mutex_unlock(&_alloc_mutex);
-		die_or_return();
-	}
 
 	b->mb_state = ST_FREE;
 	b->mb_next = NULL;
@@ -884,6 +908,12 @@ realloc(void *oldp, size_t newsize)
 #endif
 	void *newp;
 
+	if (oldp) {
+		oldb = block(oldp);
+		if (newsize <= oldb->mb_size)
+			return oldp;
+	}
+
 	newp = malloc(newsize);
 
 	if (!newp) {
@@ -957,10 +987,12 @@ malloc_dump_table(size_t minsize, size_t maxsize)
 #ifdef PARANOID
 	int any = 0;
 	int x;
+#ifdef DEBUG
 #ifdef STACKSIZE
 #ifndef GDB_HOOK
 	int sp;
 #endif
+#endif
 #endif
 	memblock_t *b;
 
