@@ -19,7 +19,6 @@
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/xpath.h>
-#include <magma.h>
 #include <ccs.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,6 +36,13 @@
 char *attr_value(resource_node_t *node, char *attrname);
 char *rg_attr_value(resource_node_t *node, char *attrname);
 
+void
+res_build_name(char *buf, size_t buflen, resource_t *res)
+{
+	snprintf(buf, buflen, "%s:%s", res->r_rule->rr_type,
+		 res->r_attrs[0].ra_value);
+}
+
 /**
    Find and determine an attribute's value. 
 
@@ -73,17 +79,27 @@ res_attr_value(resource_t *res, char *attrname)
 
    @param node		Resource tree node to look examine
    @param attrname	Attribute to retrieve.
+   @param ptype		Resource type to look for (if inheritance)
    @return 		value of attribute or NULL if not found
  */
-char *
-attr_value(resource_node_t *node, char *attrname)
+static char *
+_attr_value(resource_node_t *node, char *attrname, char *ptype)
 {
-	resource_t *res = node->rn_resource;
+	resource_t *res;
 	resource_attr_t *ra;
-	char *c, *p_type;
+	char *c, p_type[32];
 	ssize_t len;
 	int x;
 
+	if (!node)
+		return NULL;
+
+	res = node->rn_resource;
+
+	/* Go up the tree if it's not the right parent type */
+	if (ptype && strcmp(res->r_rule->rr_type, ptype))
+		return _attr_value(node->rn_parent, attrname, ptype);
+
 	for (x = 0; res->r_attrs && res->r_attrs[x].ra_name; x++) {
 		if (strcmp(attrname, res->r_attrs[x].ra_name))
 			continue;
@@ -102,28 +118,29 @@ attr_value(resource_node_t *node, char *attrname)
 		if (!c) {
 			/* Someone doesn't care or uses older
 			   semantics on inheritance */
-			return attr_value(node->rn_parent, ra->ra_value);
+			return _attr_value(node->rn_parent, ra->ra_value,
+					   NULL);
 		}
-
-		p_type = node->rn_parent->rn_resource->r_rule->rr_type;
+		
 		len = (c - ra->ra_value);
-
-		/* Different sizes? */
-		if (strlen(p_type) != len)
-			return NULL;
-
-		/* Check for parent type match */
-		if (strncmp(p_type, ra->ra_value, len) != 0)
-			return NULL;
-
+		memset(p_type, 0, sizeof(p_type));
+		memcpy(p_type, ra->ra_value, len);
+		
 		/* Skip the "%" and recurse */
-		return attr_value(node->rn_parent, ++c);
+		return _attr_value(node->rn_parent, ++c, p_type);
 	}
 
 	return NULL;
 }
 
 
+char *
+attr_value(resource_node_t *node, char *attrname)
+{
+	return _attr_value(node, attrname, NULL);
+}
+
+
 /**
   Run to the top of the tree.  Used to determine certain attributes of the
   resource group in-line, during resource tree operations.
@@ -269,11 +286,24 @@ resource_t *
 find_root_by_ref(resource_t **reslist, char *ref)
 {
 	resource_t *curr;
+	char ref_buf[128];
+	char *type;
+	char *name;
 	int x;
 
+	snprintf(ref_buf, sizeof(ref_buf), "%s", ref);
+
+	type = ref_buf;
+	if ((name = strchr(ref_buf, ':'))) {
+		*name = 0;
+		name++;
+	} else {
+		/* Default type */
+		type = "service";
+		name = ref;
+	}
+
 	list_do(reslist, curr) {
-		if (curr->r_rule->rr_root == 0)
-			continue;
 
 		/*
 		   This should be one operation - the primary attr
@@ -281,15 +311,18 @@ find_root_by_ref(resource_t **reslist, char *ref)
 		 */
 		for (x = 0; curr->r_attrs && curr->r_attrs[x].ra_name;
 		     x++) {
+			if (strcmp(type, curr->r_rule->rr_type))
+				continue;
 			if (!(curr->r_attrs[x].ra_flags & RA_PRIMARY))
 				continue;
-			if (strcmp(ref, curr->r_attrs[x].ra_value))
+			if (strcmp(name, curr->r_attrs[x].ra_value))
 				continue;
 
 			return curr;
 		}
 	} while (!list_done(reslist, curr));
 
+
 	return NULL;
 }
 
@@ -391,17 +424,52 @@ xpath_get_one(xmlDocPtr doc, xmlXPathContextPtr ctx, char *query)
 {
 	char *val = NULL, *ret = NULL;
 	xmlXPathObjectPtr obj;
+	xmlNodePtr node;
+	size_t size = 0;
+	int nnv = 0;
 
 	obj = xmlXPathEvalExpression((unsigned char *)query, ctx);
 	if (!obj)
 		return NULL;
+	if (!obj->nodesetval)
+		goto out;
+	if (obj->nodesetval->nodeNr <= 0)
+		goto out;
 
-	if (obj->nodesetval && obj->nodesetval->nodeNr >= 1) {
-		val = (char *)obj->nodesetval->nodeTab[0]->children->content;
-		if (strlen(val) >= 1)
-			ret = strdup(val);
+	node = obj->nodesetval->nodeTab[0];
+	if(!node)
+		goto out;
+
+	if (((node->type == XML_ATTRIBUTE_NODE) && strstr(query, "@*")) ||
+	    ((node->type == XML_ELEMENT_NODE) && strstr(query, "child::*"))){
+		if (node->children && node->children->content)
+	  		size = strlen((char *)node->children->content)+
+				      strlen((char *)node->name)+2;
+		else 
+			size = strlen((char *)node->name)+2;
+		nnv = 1;
+	} else {
+		if (node->children && node->children->content) {
+			size = strlen((char *)node->children->content)+1;
+		} else {
+			goto out;
+		}
 	}
 
+	val = (char *)malloc(size);
+	if(!val)
+		goto out;
+	memset(val, 0, size);
+	if (nnv) {
+		sprintf(val, "%s=%s", node->name, node->children ?
+			(char *)node->children->content:"");
+	} else {
+		sprintf(val, "%s", node->children ? node->children->content :
+			node->name);
+	}
+
+	ret = val;
+out:
 	xmlXPathFreeObject(obj);
 
 	return ret;
@@ -468,8 +536,6 @@ print_resource(resource_t *res)
 	int x;
 
 	printf("Resource type: %s", res->r_rule->rr_type);
-	if (res->r_rule->rr_root)
-		printf(" [ROOT]");
 	if (res->r_flags & RF_INLINE)
 		printf(" [INLINE]");
 	if (res->r_flags & RF_NEEDSTART)
@@ -725,6 +791,7 @@ load_resource(int ccsfd, resource_rule_t *rule, char *base)
 	}
 
 	if (!found) {
+		//printf("No attributes found for %s\n", base);
 		destroy_resource(res);
 		return NULL;
 	}
@@ -771,6 +838,7 @@ load_resources(int ccsfd, resource_t **reslist, resource_rule_t **rulelist)
 				      "Error storing %s resource\n",
 				      newres->r_rule->rr_type);
 #endif
+
 			       destroy_resource(newres);
 		       }
 
