@@ -19,10 +19,10 @@
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/xpath.h>
-#include <magma.h>
 #include <ccs.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <resgroup.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -50,31 +50,31 @@ store_rule(resource_rule_t **rulelist, resource_rule_t *newrule)
 	resource_rule_t *curr;
 
 	list_do(rulelist, curr) {
-		if (!strcmp(newrule->rr_type, curr->rr_type)) {
+		if (!strcasecmp(newrule->rr_type, curr->rr_type)) {
 #ifdef NO_CCS
 			fprintf(stderr, "Error storing %s: Duplicate\n",
 				newrule->rr_type);
 #else
 			clulog(LOG_ERR, "Error storing %s: Duplicate\n",
-				newrule->rr_type);
-#endif
-			return -1;
-		}
-		if (newrule->rr_root && curr->rr_root) {
-#ifdef NO_CCS
-			fprintf(stderr, "Error storing %s: root "
-				"resource type %s exists already\n",
-				newrule->rr_type, curr->rr_type);
-#else
-			clulog(LOG_ERR, "Error storing %s: root "
-				"resource type %s exists already\n",
-				newrule->rr_type, curr->rr_type);
+			       newrule->rr_type);
 #endif
 			return -1;
 		}
 
 	} while (!list_done(rulelist, curr));
-			
+	
+	/* insert sorted in alphabetical order so rg_test produces
+	 * reproducible output all the time */
+	list_do(rulelist, curr) {
+		if (strcasecmp(newrule->rr_type, curr->rr_type) < 0) {
+			list_insert(&curr, newrule);
+			/* reset list if we have a new low */
+			if (curr == *rulelist)
+				*rulelist = newrule;
+			return 0;
+		}
+	} while (!list_done(rulelist, curr));
+		
 	list_insert(rulelist, newrule);
 	return 0;
 }
@@ -199,30 +199,6 @@ _get_version(xmlDocPtr doc, xmlXPathContextPtr ctx, char *base,
 }
 
 
-/**
-   Get and store the root attribute.
-
-   @param doc		Pre-parsed XML document pointer.
-   @param ctx		Pre-allocated XML XPath context pointer.
-   @param base		XPath prefix to search
-   @param rr		Resource rule to store new information in.
- */
-void
-_get_root(xmlDocPtr doc, xmlXPathContextPtr ctx, char *base,
-	  resource_rule_t *rr)
-{
-	char xpath[256];
-	char *ret = NULL;
-
-	snprintf(xpath, sizeof(xpath), "%s/attributes/@root", base);
-	ret = xpath_get_one(doc, ctx, xpath);
-	if (ret) {
-		rr->rr_root = 1;
-		free(ret);
-	}
-}
-
-
 int
 expand_time(char *val)
 {
@@ -298,6 +274,7 @@ store_action(resource_act_t **actsp, char *name, int depth,
 		acts[0].ra_depth = depth;
 		acts[0].ra_timeout = timeout;
 		acts[0].ra_interval = interval;
+		acts[0].ra_last = 0;
 		acts[1].ra_name = NULL;
 
 		*actsp = acts;
@@ -321,6 +298,7 @@ store_action(resource_act_t **actsp, char *name, int depth,
 				       (int)interval);
 				acts[x].ra_interval = interval;
 			}
+			acts[x].ra_last = 0;
 			printf("\n");
 			replace = 1;
 		}
@@ -342,6 +320,7 @@ store_action(resource_act_t **actsp, char *name, int depth,
 	acts[x].ra_depth = depth;
 	acts[x].ra_timeout = timeout;
 	acts[x].ra_interval = interval;
+	acts[x].ra_last = 0;
 
 	acts[x+1].ra_name = NULL;
 
@@ -409,8 +388,6 @@ _get_actions(xmlDocPtr doc, xmlXPathContextPtr ctx, char *base,
 				 interval) != 0)
 			free(act);
 	} while (1);
-
-
 }
 
 
@@ -478,48 +455,6 @@ store_attribute(resource_attr_t **attrsp, char *name, char *value, int flags)
 }
 
 
-/**
-  Take the first unique + required attr and call it the 'primary' attr
-  for rgmanager.  If there's no primary, index 0 becomes the primary attr.
- */
-int
-choose_primary(resource_attr_t *attrs)
-{
-	int x = 0, primary = 0;
-	int flags;
-	char *name, *val;
-
-	if (!attrs)
-		return 0;
-
-	for (x = 0; attrs[x].ra_name; x++) {
-
-		if ((attrs[x].ra_flags & (RA_UNIQUE | RA_REQUIRED)) == 
-		    (RA_UNIQUE | RA_REQUIRED)) {
-			primary = x;
-			break;
-		}
-	}
-
-	if (primary != 0) {
-		flags = attrs[primary].ra_flags | RA_PRIMARY;
-		name = attrs[primary].ra_name;
-		val = attrs[primary].ra_value;
-
-		attrs[primary].ra_flags = attrs[0].ra_flags;
-		attrs[primary].ra_name = attrs[0].ra_name;
-		attrs[primary].ra_value = attrs[0].ra_value;
-
-		attrs[0].ra_flags = flags;
-		attrs[0].ra_name = name;
-		attrs[0].ra_value = val;
-	} else {
-		attrs[0].ra_flags |= RA_PRIMARY;
-	}
-
-	return 0;
-}
-
 /**
    Store a child type in the child array of a resource rule.
    XXX Could be rewritten to use list macros.
@@ -529,11 +464,12 @@ choose_primary(resource_attr_t *attrs)
    @param start		Start level
    @param stop		Stop level
    @param forbid	Do NOT allow this child type to exist
+   @param flags		set to 1 to note that it was defined inline
    @return		0 on success, nonzero on failure
  */
 int
 store_childtype(resource_child_t **childp, char *name, int start, int stop,
-		int forbid)
+		int forbid, int flags)
 {
 	int x = 0;
 	resource_child_t *child = *childp;
@@ -549,6 +485,7 @@ store_childtype(resource_child_t **childp, char *name, int start, int stop,
 		child[0].rc_startlevel = start;
 		child[0].rc_stoplevel = stop;
 		child[0].rc_forbid = forbid;
+		child[0].rc_flags = flags;
 		child[1].rc_name = NULL;
 
 		*childp = child;
@@ -565,6 +502,7 @@ store_childtype(resource_child_t **childp, char *name, int start, int stop,
 	child[x].rc_startlevel = start;
 	child[x].rc_stoplevel = stop;
 	child[x].rc_forbid = forbid;
+	child[x].rc_flags = flags;
 	child[x+1].rc_name = NULL;
 
 	*childp = child;
@@ -582,10 +520,7 @@ print_resource_rule(resource_rule_t *rr)
 {
 	int x;
 
-	printf("Resource Rules for \"%s\"", rr->rr_type);
-	if (rr->rr_root)
-		printf(" [ROOT]");
-	printf("\n");
+	printf("Resource Rules for \"%s\"\n", rr->rr_type);
 
 	if (rr->rr_version)
 		printf("OCF API Version: %s\n", rr->rr_version);
@@ -792,9 +727,6 @@ _get_rule_attrs(xmlDocPtr doc, xmlXPathContextPtr ctx, char *base,
 			store_attribute(&rr->rr_attrs, attrname, ret, flags);
 	}
 
-	if (!primary_found)
-		choose_primary(rr->rr_attrs);
-
 	return 0;
 }
 
@@ -863,7 +795,7 @@ _get_childtypes(xmlDocPtr doc, xmlXPathContextPtr ctx, char *base,
 		 */
 		if (childname)
 			store_childtype(&rr->rr_childtypes, childname,
-					startlevel, stoplevel, forbid);
+					startlevel, stoplevel, forbid, 0);
 	}
 
 	return 0;
@@ -886,6 +818,10 @@ read_pipe(int fd, char **file, size_t *length)
 
 		n = read(fd, buf, sizeof(buf));
 		if (n < 0) {
+
+			if (errno == EINTR)
+				continue;
+
 			if (*file)
 				free(*file);
 			return -1;
@@ -1030,12 +966,11 @@ load_resource_rulefile(char *filename, resource_rule_t **rules)
 		snprintf(base, sizeof(base),
 			 "/resource-agent[%d]/special[@tag=\"rgmanager\"]",
 			 ruleid);
-		_get_root(doc, ctx, base, rr);
 		_get_maxparents(doc, ctx, base, rr);
 		rr->rr_agent = strdup(filename);
 
 		/*
-		   Second, add the allowable-children fields
+		   Second, add the children fields
 		 */
 		_get_childtypes(doc, ctx, base, rr);
 
@@ -1090,8 +1025,9 @@ load_resource_rules(const char *rpath, resource_rule_t **rules)
 {
 	DIR *dir;
 	struct dirent *de;
-	char *fn;//, *dot;
+	char *fn, *dot;
 	char path[2048];
+	struct stat st_buf;
 
 	dir = opendir(rpath);
 	if (!dir)
@@ -1104,14 +1040,36 @@ load_resource_rules(const char *rpath, resource_rule_t **rules)
 		if (!fn)
 			continue;
 		
+		/* Ignore files with common backup extension */
 		if ((fn != NULL) && (strlen(fn) > 0) && 
 			(fn[strlen(fn)-1] == '~')) 
 			continue;
 
+ 		dot = strrchr(fn, '.');
+ 		if (dot) {
+ 			/* Ignore RPM installed save files, patches,
+ 			   diffs, etc. */
+ 			if (!strncasecmp(dot, ".rpm", 4)) {
+ 				fprintf(stderr, "Warning: "
+ 					"Ignoring %s/%s: Bad extension %s\n",
+ 					rpath, de->d_name, dot);
+ 				continue;
+ 			}
+ 		}
+
 		snprintf(path, sizeof(path), "%s/%s",
 			 rpath, de->d_name);
-
-		load_resource_rulefile(path, rules);
+		
+		if (stat(path, &st_buf))
+			continue;
+		
+		if (S_ISDIR(st_buf.st_mode))
+			continue;
+		
+  		if (st_buf.st_mode & (S_IXUSR|S_IXOTH|S_IXGRP)) {
+  			printf("Loading resource rule from %s\n", path);
+   			load_resource_rulefile(path, rules);
+  		}
 	}
 	xmlCleanupParser();
 
