@@ -129,7 +129,7 @@ meta_data()
         <action name="monitor" depth="10" interval="60" timeout="20"/>
 
         <action name="meta-data" timeout="20"/>
-        <action name="verify-all" timeout="20"/>
+        <action name="validate-all" timeout="20"/>
     </actions>
 
     <special tag="rgmanager">
@@ -698,6 +698,7 @@ ipv4()
 			if [ "${addr/\/*/}" != "$ifaddr" ]; then
 			        continue
 			fi
+			addr=`/sbin/ip addr list | grep "$addr" | head -n 1 | awk '{print $2}'`
 			ocf_log info "Removing IPv4 address $addr from $dev"
 		fi
 		
@@ -918,7 +919,7 @@ stop)
 	;;
 status|monitor)
 	ip_op ${OCF_RESKEY_family} status ${OCF_RESKEY_address}
-	[ $? -ne 0 ] && exit $OCF_ERR_GENERIC
+	[ $? -ne 0 ] && exit $OCF_NOT_RUNNING
 	
 	check_interface_up ${OCF_RESKEY_family} ${OCF_RESKEY_address}
 	exit $?
@@ -932,13 +933,13 @@ meta-data)
 	meta_data
 	exit 0
 	;;
-verify-all)
+validate-all|verify_all)
 	verify_all
 	exit $?
 	;;
 *)
-	echo "usage: $0 {start|stop|status|monitor|restart|meta-data|verify-alll}"
-	exit $OCF_ERR_GENERIC
+	echo "usage: $0 {start|stop|status|monitor|restart|meta-data|validate-all}"
+	exit $OCF_ERR_UNIMPLEMENTED
 	;;
 esac
 
