@@ -1,13 +1,16 @@
-#include <magma.h>
-#include <magmamsg.h>
+#include <members.h>
+#include <message.h>
 #include <msgsimple.h>
 #include <resgroup.h>
 #include <platform.h>
 #include <libgen.h>
 #include <ncurses.h>
 #include <term.h>
+#include <rg_types.h>
 #include <termios.h>
 #include <ccs.h>
+#include <libcman.h>
+#include <signal.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -17,6 +20,7 @@
 #define FLAG_LOCAL 0x2
 #define FLAG_RGMGR 0x4
 #define FLAG_NOCFG 0x8	/* Shouldn't happen */
+#define FLAG_QDISK 0x10
 
 #define RG_VERBOSE 0x1
 
@@ -40,38 +44,30 @@ typedef struct {
 } rg_state_list_t;
 
 
-rg_state_list_t *
-rg_state_list(uint64_t local_node_id, int fast)
+void
+flag_rgmanager_nodes(cluster_member_list_t *cml)
 {
-	int fd, n, x;
-	rg_state_list_t *rsl = NULL;
-	generic_msg_hdr *msgp = NULL;
-	rg_state_msg_t *rsmp = NULL;
+	msgctx_t ctx;
+	int max = 0, n;
+	generic_msg_hdr *msgp;
 	fd_set rfds;
+
 	struct timeval tv;
 
-	fd = msg_open(local_node_id, RG_PORT, RG_PURPOSE, 10);
-	if (fd == -1) {
+	if (msg_open(MSG_SOCKET, 0, 0, &ctx, 10) < 0) {
 		perror("msg_open");
-		return NULL;
+		return;
 	}
 
-	msg_send_simple(fd, RG_STATUS, fast, 0);
-
-	rsl = malloc(sizeof(rg_state_list_t));
-	if (!rsl) {
-		printf("Try again, out of memory\n");
-		exit(0);
-	}
-	memset(rsl, 0, sizeof(rg_state_list_t));
+	msg_send_simple(&ctx, RG_STATUS_NODE, 0, 0);
 
 	while (1) {
 		FD_ZERO(&rfds);
-		FD_SET(fd, &rfds);
+		msg_fd_set(&ctx, &rfds, &max);
 		tv.tv_sec = 10;
 		tv.tv_usec = 0;
 
-		n = select(fd+1, &rfds, NULL, NULL, &tv);
+		n = select(max+1, &rfds, NULL, NULL, &tv);
 		if (n == 0) {
 			fprintf(stderr, "Timed out waiting for a response "
 				"from Resource Group Manager\n");
@@ -88,7 +84,8 @@ rg_state_list(uint64_t local_node_id, int fast)
 			break;
 		}
 
-		n = msg_receive_simple(fd, &msgp, tv.tv_sec);
+		n = msg_receive_simple(&ctx, &msgp, tv.tv_sec);
+
 		if (n < 0) {
 			if (errno == EAGAIN)
 				continue;
@@ -106,17 +103,123 @@ rg_state_list(uint64_t local_node_id, int fast)
 		}
 
 		swab_generic_msg_hdr(msgp);
+
+		if (msgp->gh_command == RG_FAIL) {
+			printf("Member states unavailable: %s\n", 
+			       rg_strerror(msgp->gh_arg1));
+			free(msgp);
+			msg_close(&ctx);
+			return;
+		}	
+
+		if (msgp->gh_command == RG_SUCCESS) {
+			free(msgp);
+			break;
+		}
+
+		for (n = 0; n < cml->cml_count; n++) {
+			if (cml->cml_members[n].cn_nodeid != msgp->gh_arg1)
+				continue;
+			cml->cml_members[n].cn_member |= FLAG_RGMGR;
+		}
+
+		free(msgp);
+		msgp = NULL;
+	}
+
+	msg_send_simple(&ctx, RG_SUCCESS, 0, 0);
+	msg_close(&ctx);
+
+	return;
+}
+
+
+rg_state_list_t *
+rg_state_list(int local_node_id, int fast)
+{
+	msgctx_t ctx;
+	int max = 0, n, x;
+	rg_state_list_t *rsl = NULL;
+	generic_msg_hdr *msgp = NULL;
+	rg_state_msg_t *rsmp = NULL;
+	fd_set rfds;
+
+	struct timeval tv;
+
+	if (msg_open(MSG_SOCKET, 0, 0, &ctx, 10) < 0) {
+		perror("msg_open");
+		return NULL;
+	}
+
+	msg_send_simple(&ctx, RG_STATUS, fast, 0);
+
+	rsl = malloc(sizeof(rg_state_list_t));
+	if (!rsl) {
+		printf("Try again, out of memory\n");
+		exit(0);
+	}
+	memset(rsl, 0, sizeof(rg_state_list_t));
+
+	while (1) {
+		FD_ZERO(&rfds);
+		msg_fd_set(&ctx, &rfds, &max);
+		tv.tv_sec = 10;
+		tv.tv_usec = 0;
+
+		n = select(max+1, &rfds, NULL, NULL, &tv);
+		if (n == 0) {
+			fprintf(stderr, "Timed out waiting for a response "
+				"from Resource Group Manager\n");
+			break;
+		}
+
+		if (n < 0) {
+			if (errno == EAGAIN ||
+			    errno == EINTR)
+				continue;
+			fprintf(stderr, "Failed to receive "
+				"service data: select: %s\n",
+				strerror(errno));
+			break;
+		}
+
+		n = msg_receive_simple(&ctx, &msgp, tv.tv_sec);
+
+		if (n < 0) {
+			if (errno == EAGAIN)
+				continue;
+			perror("msg_receive_simple");
+			break;
+		}
+	        if (n < sizeof(generic_msg_hdr)) {
+			printf("Error: Malformed message\n");
+			break;
+		}
+
+		if (!msgp) {
+			printf("Error: no message?!\n");
+			break;
+		}
+
+		swab_generic_msg_hdr(msgp);
+
+		if (msgp->gh_command == RG_FAIL) {
+			printf("Service states unavailable: %s\n", 
+			       rg_strerror(msgp->gh_arg1));
+			msg_close(&ctx);
+			return NULL;
+		}	
+
 		if (msgp->gh_command == RG_SUCCESS) {
 			free(msgp);
 			break;
 		}
 
 		if (n < sizeof(*rsmp)) {
-			msg_close(fd);
+			msg_close(&ctx);
 			return NULL;
 		}
 
-
 		rsmp = (rg_state_msg_t *)msgp;
 
 		swab_rg_state_t(&rsmp->rsm_state);
@@ -137,8 +240,8 @@ rg_state_list(uint64_t local_node_id, int fast)
 		msgp = NULL;
 	}
 
-	msg_send_simple(fd, RG_SUCCESS, 0, 0);
-	msg_close(fd);
+	msg_send_simple(&ctx, RG_SUCCESS, 0, 0);
+	msg_close(&ctx);
 
 	if (!rsl->rgl_count) {
 		free(rsl);
@@ -156,46 +259,67 @@ cluster_member_list_t *ccs_member_list(void)
 	char buf[128];
 	char *name;
 	cluster_member_list_t *ret = NULL;
+	cman_node_t *nodes = NULL;
 
 	desc = ccs_connect();
 	if (desc < 0) {
 		return NULL;
 	}
 
-	x = 1;
+	while ((ret = malloc(sizeof(*ret))) == NULL)
+		sleep(1);
 	
-	snprintf(buf, sizeof(buf),
-		 "/cluster/clusternodes/clusternode[%d]/@name", x);
-	while (ccs_get(desc, buf, &name) == 0) {
-		if (!ret) {
-			ret = malloc(cml_size(x));
-			if (!ret) {
+	x = 0;
+	while (++x) {
+		name = NULL;
+		snprintf(buf, sizeof(buf),
+			"/cluster/clusternodes/clusternode[%d]/@name", x);
+
+		if (ccs_get(desc, buf, &name) != 0)
+			break;
+
+		if (!name)
+			break;
+		if (!strlen(name)) {
+			free(name);
+			continue;
+		}
+
+		if (!nodes) {
+			nodes = malloc(x * sizeof(cman_node_t));
+			if (!nodes) {
 				perror("malloc");
 				ccs_disconnect(desc);
 				exit(1);
 			}
-			memset(ret, 0, cml_size(x));
 		} else {
-			ret = realloc(ret, cml_size(x));
-			if (!ret) {
+			nodes = realloc(nodes, x * sizeof(cman_node_t));
+			if (!nodes) {
 				perror("realloc");
 				ccs_disconnect(desc);
 				exit(1);
 			}
 		}
 
-		memset(&ret->cml_members[x-1], 0, sizeof(cluster_member_t));
-		strncpy(ret->cml_members[x-1].cm_name, name,
-			sizeof(ret->cml_members[x-1].cm_name));
+		memset(&nodes[x-1], 0, sizeof(cman_node_t));
+		strncpy(nodes[x-1].cn_name, name,
+			sizeof(nodes[x-1].cn_name));
 		free(name);
 
-		ret->cml_count = x;
-		++x;
+		/* Add node ID */
 		snprintf(buf, sizeof(buf),
-			 "/cluster/clusternodes/clusternode[%d]/@name", x);
+			 "/cluster/clusternodes/clusternode[%d]/@nodeid", x);
+		if (ccs_get(desc, buf, &name) == 0) {
+			nodes[x-1].cn_nodeid = atoi(name);
+			free(name);
+		}
+
+		ret->cml_count = x;
 	}
 
 	ccs_disconnect(desc);
+	ret->cml_members = nodes;
+
 	return ret;
 }
 
@@ -205,15 +329,15 @@ flag_nodes(cluster_member_list_t *all, cluster_member_list_t *these,
 	   uint8_t flag)
 {
 	int x;
-	cluster_member_t *m;
+	cman_node_t *m;
 
 	for (x=0; x<all->cml_count; x++) {
 
-		m = memb_name_to_p(these, all->cml_members[x].cm_name);
+		m = memb_name_to_p(these, all->cml_members[x].cn_name);
 
-		if (m) {
-			all->cml_members[x].cm_id = m->cm_id;
-			all->cml_members[x].cm_state |= flag;
+		if (m && m->cn_member) {
+			all->cml_members[x].cn_nodeid = m->cn_nodeid;
+			all->cml_members[x].cn_member |= flag;
 		}
 	}
 }
@@ -222,34 +346,45 @@ flag_nodes(cluster_member_list_t *all, cluster_member_list_t *these,
 cluster_member_list_t *
 add_missing(cluster_member_list_t *all, cluster_member_list_t *these)
 {
-	int x, y;
-	cluster_member_t *m, *new;
+	int x, y, addflag;
+	cman_node_t *m, *nn;
+
+	if (!these)
+		return all;
 
 	for (x=0; x<these->cml_count; x++) {
 
 		m = NULL;
         	for (y = 0; y < all->cml_count; y++) {
-			if (!strcmp(all->cml_members[y].cm_name,
-				    these->cml_members[x].cm_name))
+			if (!strcmp(all->cml_members[y].cn_name,
+				    these->cml_members[x].cn_name))
                         	m = &all->cml_members[y];
 		}
-
+        	
 		if (!m) {
-			all = realloc(all, cml_size((all->cml_count+1)));
-			if (!all) {
+			all->cml_members = realloc(all->cml_members,
+						   (all->cml_count+1) *
+						   sizeof(cman_node_t));
+			if (!all->cml_members) {
 				perror("realloc");
 				exit(1);
 			}
 			
-			new = &all->cml_members[all->cml_count];
+			nn = &all->cml_members[all->cml_count];
 
-			memcpy(new, &these->cml_members[x],
-			       sizeof(cluster_member_t));
-
-			if (new->cm_state == STATE_UP) {
-				new->cm_state = FLAG_UP | FLAG_NOCFG;
+			memcpy(nn, &these->cml_members[x],
+			       sizeof(cman_node_t));
+			
+			if (nn->cn_nodeid == 0) { /* quorum disk? */
+				addflag = FLAG_QDISK;
 			} else {
-				new->cm_state = FLAG_NOCFG;
+				addflag = FLAG_NOCFG;
+			}
+
+			if (nn->cn_member) {
+				nn->cn_member = FLAG_UP | addflag;
+			} else {
+				nn->cn_member = addflag;
 			}
 			++all->cml_count;
 
@@ -261,16 +396,16 @@ add_missing(cluster_member_list_t *all, cluster_member_list_t *these)
 
 
 char *
-my_memb_id_to_name(cluster_member_list_t *members, uint64_t memb_id)
+my_memb_id_to_name(cluster_member_list_t *members, int memb_id)
 {
 	int x;
 
-	if (memb_id == NODE_ID_NONE)
+	if (memb_id == 0)
 		return "none";
 
 	for (x = 0; x < members->cml_count; x++) {
-		if (members->cml_members[x].cm_id == memb_id)
-			return members->cml_members[x].cm_name;
+		if (members->cml_members[x].cn_nodeid == memb_id)
+			return members->cml_members[x].cn_name;
 	}
 
 	return "unknown";
@@ -281,7 +416,7 @@ void
 _txt_rg_state(rg_state_t *rs, cluster_member_list_t *members, int flags)
 {
 	char owner[31];
-
+	char flags_string[255] = "";
 
 	if (rs->rs_state == RG_STATE_STOPPED ||
 	    rs->rs_state == RG_STATE_DISABLED ||
@@ -295,19 +430,34 @@ _txt_rg_state(rg_state_t *rs, cluster_member_list_t *members, int flags)
 		snprintf(owner, sizeof(owner), "%-.30s",
 			 my_memb_id_to_name(members, rs->rs_owner));
 	}
-	printf("  %-20.20s %-30.30s %-16.16s\n",
+	rg_flags_str(flags_string, sizeof(flags_string), rs->rs_flags, ", ");
+	printf("  %-20.20s %-30.30s %-16.16s ",
 	       rs->rs_name,
 	       owner,
 	       rg_state_str(rs->rs_state));
+	if(strlen(flags_string))
+		printf ("%-30.30s\n", flags_string);
+	else
+		printf("\n");
 }
 
 
 void
 _txt_rg_state_v(rg_state_t *rs, cluster_member_list_t *members, int flags)
 {
+	char flags_string[255] = "";
+
+	rg_flags_str(flags_string, sizeof(flags_string), rs->rs_flags, ", ");
+
 	printf("Service Name      : %s\n", rs->rs_name);
 	printf("  Current State   : %s (%d)\n",
 	       rg_state_str(rs->rs_state), rs->rs_state);
+	if (rs->rs_flags)
+		printf("  Flags           : %s (%d)\n",
+		       flags_string, rs->rs_flags);
+	else
+		printf("  Flags           : none (%d)\n",
+		       rs->rs_flags);
 	printf("  Owner           : %s\n",
 	       my_memb_id_to_name(members, rs->rs_owner));
 	printf("  Last Owner      : %s\n",
@@ -331,6 +481,7 @@ void
 xml_rg_state(rg_state_t *rs, cluster_member_list_t *members, int flags)
 {
 	char time_str[32];
+	char flags_string[255] = "";
 	int x;
 
 	/* Chop off newlines */
@@ -342,12 +493,15 @@ xml_rg_state(rg_state_t *rs, cluster_member_list_t *members, int flags)
 		}
 	}
 
-	printf("    <group name=\"%s\" state=\"%d\" state_str=\"%s\" "
+	printf("    <group name=\"%s\" state=\"%d\" state_str=\"%s\""
+	       " flags=\"%d\" flags_str=\"%s\""
 	       " owner=\"%s\" last_owner=\"%s\" restarts=\"%d\""
 	       " last_transition=\"%llu\" last_transition_str=\"%s\"/>\n",
 	       rs->rs_name,
 	       rs->rs_state,
 	       rg_state_str(rs->rs_state),
+	       rs->rs_flags,
+	       rg_flags_str(flags_string, sizeof(flags_string), rs->rs_flags, " "),
 	       my_memb_id_to_name(members, rs->rs_owner),
 	       my_memb_id_to_name(members, rs->rs_last_owner),
 	       rs->rs_restarts,
@@ -365,11 +519,14 @@ txt_rg_states(rg_state_list_t *rgl, cluster_member_list_t *members,
 	if (!rgl || !members)
 		return -1;
 
+	if (svcname)
+		ret = -1;
+
 	if (!(flags & RG_VERBOSE)) {
-		printf("  %-20.20s %-30.30s %-14.14s\n",
-		       "Service Name", "Owner (Last)", "State");
-		printf("  %-20.20s %-30.30s %-14.14s\n",
-		       "------- ----", "----- ------", "-----");
+		printf("  %-20.20s %-30.30s %-16.16s %-30.30s\n",
+		       "Service Name", "Owner (Last)", "State", "Flags");
+		printf("  %-20.20s %-30.30s %-16.16s %-30.30s\n",
+		       "------- ----", "----- ------", "-----", "-----");
 	} else {
 		printf("Service Information\n"
 		       "------- -----------\n\n");
@@ -385,6 +542,7 @@ txt_rg_states(rg_state_list_t *rgl, cluster_member_list_t *members,
 			case RG_STATE_STARTING:
 			case RG_STATE_STARTED:
 			case RG_STATE_STOPPING:
+				ret = 0;
 				break;
 			default:
 				ret = rgl->rgl_states[x].rs_state;
@@ -435,9 +593,9 @@ txt_quorum_state(int qs)
 {
 	printf("Member Status: ");
 
-	if (qs & QF_QUORATE)
+	if (qs) {
 		printf("Quorate\n\n");
-	else {
+	} else {
 		printf("Inquorate\n\n");
 	}
 }
@@ -446,40 +604,45 @@ txt_quorum_state(int qs)
 void
 xml_quorum_state(int qs)
 {
+	/* XXX output groupmember attr (carry over from RHCS4) */
 	printf("  <quorum ");
 
-	if (qs & QF_QUORATE)
+	if (qs & FLAG_UP) {
 		printf("quorate=\"1\"");
-	else {
-		printf("quorate=\"0\" groupmember=\"0\"/>\n");
-		return;
+	} else {
+		printf("quorate=\"0\"\n");
 	}
-
-	if (qs & QF_GROUPMEMBER)
+	if (qs & FLAG_RGMGR) {
 		printf(" groupmember=\"1\"");
-
+	} else {
+		printf(" groupmember=\"0\"");
+	}
 	printf("/>\n");
 }
 
 
 void
-txt_member_state(cluster_member_t *node)
+txt_member_state(cman_node_t *node)
 {
-	printf("  %-40.40s ", node->cm_name);
+	printf("  %-34.34s %4d ", node->cn_name,
+	       node->cn_nodeid);
 
-	if (node->cm_state & FLAG_UP)
+	if (node->cn_member & FLAG_UP)
 		printf("Online");
 	else
 		printf("Offline");
 
-	if (node->cm_state & FLAG_LOCAL)
+	if (node->cn_member & FLAG_LOCAL)
 		printf(", Local");
 	
-	if (node->cm_state & FLAG_NOCFG)
+	if (node->cn_member & FLAG_NOCFG)
 		printf(", Estranged");
-
-	if (node->cm_state & FLAG_RGMGR)
+	
+	if (node->cn_member & FLAG_RGMGR)
 		printf(", rgmanager");
+	
+	if (node->cn_member & FLAG_QDISK)
+		printf(", Quorum Disk");
 
 	printf("\n");
 		
@@ -488,38 +651,38 @@ txt_member_state(cluster_member_t *node)
 
 
 void
-xml_member_state(cluster_member_t *node)
+xml_member_state(cman_node_t *node)
 {
 	printf("    <node name=\"%s\" state=\"%d\" local=\"%d\" "
-	       "estranged=\"%d\" rgmanager=\"%d\" nodeid=\"0x%08x%08x\"/>\n",
-	       node->cm_name,
-	       !!(node->cm_state & FLAG_UP),
-	       !!(node->cm_state & FLAG_LOCAL),
-	       !!(node->cm_state & FLAG_NOCFG),
-	       !!(node->cm_state & FLAG_RGMGR),
-	       (uint32_t)((node->cm_id >> 32)&0xffffffff),
-	       (uint32_t)((node->cm_id      )&0xffffffff));
+	       "estranged=\"%d\" rgmanager=\"%d\" qdisk=\"%d\" nodeid=\"0x%08x\"/>\n",
+	       node->cn_name,
+	       !!(node->cn_member & FLAG_UP),
+	       !!(node->cn_member & FLAG_LOCAL),
+	       !!(node->cn_member & FLAG_NOCFG),
+	       !!(node->cn_member & FLAG_RGMGR),
+	       !!(node->cn_member & FLAG_QDISK),
+	       (uint32_t)((node->cn_nodeid      )&0xffffffff));
 }
 
 
-int 
+int
 txt_member_states(cluster_member_list_t *membership, char *name)
 {
 	int x, ret = 0;
 
-	if (!membership) {
-		printf("Membership information not available\n");
-		return -1;
-	}
+  	if (!membership) {
+  		printf("Membership information not available\n");
+ 		return -1;
+  	}
 
-	printf("  %-40.40s %s\n", "Member Name", "Status");
-	printf("  %-40.40s %s\n", "------ ----", "------");
+	printf("  %-34.34s %-4.4s %s\n", "Member Name", "ID", "Status");
+	printf("  %-34.34s %-4.4s %s\n", "------ ----", "----", "------");
 
 	for (x = 0; x < membership->cml_count; x++) {
-		if (name && strcmp(membership->cml_members[x].cm_name, name))
+		if (name && strcmp(membership->cml_members[x].cn_name, name))
 			continue;
 		txt_member_state(&membership->cml_members[x]);
-		ret = !(membership->cml_members[x].cm_state & FLAG_UP);
+ 		ret = !(membership->cml_members[x].cn_member & FLAG_UP);
 	}
 
 	printf("\n");
@@ -527,7 +690,7 @@ txt_member_states(cluster_member_list_t *membership, char *name)
 }
 
 
-int 
+int
 xml_member_states(cluster_member_list_t *membership, char *name)
 {
 	int x, ret = 0;
@@ -539,11 +702,11 @@ xml_member_states(cluster_member_list_t *membership, char *name)
 
 	printf("  <nodes>\n");
 	for (x = 0; x < membership->cml_count; x++) {
-		if (name && strcmp(membership->cml_members[x].cm_name, name))
+		if (name && strcmp(membership->cml_members[x].cn_name, name))
 			continue;
 		xml_member_state(&membership->cml_members[x]);
 		if (name)
-			ret = !(membership->cml_members[x].cm_state & FLAG_UP);
+			ret = !(membership->cml_members[x].cn_member & FLAG_UP);
 	}
 	printf("  </nodes>\n");
 	
@@ -560,19 +723,20 @@ txt_cluster_status(int qs, cluster_member_list_t *membership,
 	
 	if (!svcname && !name) {
 		txt_quorum_state(qs);
-		if (!membership || !(qs & QF_GROUPMEMBER)) {
+		if (!membership) {
+			/* XXX Check for rgmanager?! */
 			printf("Resource Group Manager not running; "
 			       "no service information available.\n\n");
 		}
 	}
 
-	if (!svcname || (name && svcname))
-		ret = txt_member_states(membership, name);
-	if (name && !svcname)
-		return ret;
-	if (!name || (name && svcname))
-		ret = txt_rg_states(rgs, membership, svcname, flags);
-	return ret;
+  	if (!svcname || (name && svcname))
+ 		ret = txt_member_states(membership, name);
+ 	if (name && !svcname)
+ 		return ret;
+ 	if (!name || (name && svcname))
+ 		ret = txt_rg_states(rgs, membership, svcname, flags);
+ 	return ret;
 }
 
 
@@ -581,34 +745,49 @@ xml_cluster_status(int qs, cluster_member_list_t *membership,
 		   rg_state_list_t *rgs, char *name, char *svcname,
 		   int flags)
 {
-	int ret1 = 0, ret2 = -1;
-	
+ 	int ret1 = 0, ret2 = -1;
+	int x;
+
 	printf("<?xml version=\"1.0\"?>\n");
 	printf("<clustat version=\"4.1.1\">\n");
 
-	if (!svcname && !name)
-		xml_quorum_state(qs);
-	if (!svcname || (name && svcname)) 
-		ret1 = xml_member_states(membership, name);
-	
-	if (rgs &&
-	    (!name || (name && svcname)))
-		ret2 = xml_rg_states(rgs, membership, svcname, flags);
-	printf("</clustat>\n");
-	
-	if (name && ret1)
-		return ret1;
-	if (svcname && ret2)
-		return ret2;
-	return 0;
+	if (qs) {
+		qs = FLAG_UP;
+		if (membership) {
+			for (x = 0; x < membership->cml_count; x++) {
+				if ((membership->cml_members[x].cn_member &
+				     (FLAG_LOCAL|FLAG_RGMGR)) ==
+				     (FLAG_LOCAL|FLAG_RGMGR)) {
+					qs |= FLAG_RGMGR;
+					break;
+				}
+			}
+		}
+	}
+
+  	if (!svcname && !name)
+  		xml_quorum_state(qs);
+  	if (!svcname || (name && svcname)) 
+ 		ret1 = xml_member_states(membership, name);
+ 	
+  	if (rgs &&
+  	    (!name || (name && svcname)))
+ 		ret2 = xml_rg_states(rgs, membership, svcname, flags);
+  	printf("</clustat>\n");
+ 	
+ 	if (name && ret1)
+ 		return ret1;
+ 	if (svcname && ret2)
+ 		return ret2;
+ 	return 0;
 }
 
 
 cluster_member_list_t *
-build_member_list(uint64_t *lid)
+build_member_list(cman_handle_t ch, int *lid)
 {
 	cluster_member_list_t *all, *part;
-	cluster_member_t *m;
+	cman_node_t *m;
 	int root = 0;
 	int x;
 
@@ -616,8 +795,7 @@ build_member_list(uint64_t *lid)
 	   infrastructure */
 	root = (getuid() == 0 || geteuid() == 0);
 
-	part = clu_member_list(NULL);
-	msg_update(part); /* XXX magmamsg is awful. */
+	part = get_member_list(ch);
 
 	if (root && (all = ccs_member_list())) {
 
@@ -627,33 +805,29 @@ build_member_list(uint64_t *lid)
 
 		/* Flag online nodes */
 		flag_nodes(all, part, FLAG_UP);
-
-		cml_free(part);
+		free_member_list(part);
 	} else {
 		/* not root - keep it simple for the next block */
 		all = part;
 	}
 
+	if (!all) {
+		*lid = 0;
+		return NULL;
+	}
+
 	/* Grab the local node ID and flag it from the list of reported
 	   online nodes */
-	clu_local_nodeid(NULL, lid);
-
-	if (!all)
-		return NULL;
-
+	*lid = get_my_nodeid(ch);
+	/* */
 	for (x=0; x<all->cml_count; x++) {
-		if (all->cml_members[x].cm_id == *lid) {
+		if (all->cml_members[x].cn_nodeid == *lid) {
 			m = &all->cml_members[x];
-			m->cm_state |= FLAG_LOCAL;
+			m->cn_member |= FLAG_LOCAL;
 			break;
 		}
 	}
 
-	/* Flag rgmanager nodes, if any */
-	part = clu_member_list(RG_SERVICE_GROUP);
-	flag_nodes(all, part, FLAG_RGMGR);
-	cml_free(part);
-
 	return all;
 }
 
@@ -683,14 +857,15 @@ main(int argc, char **argv)
 	int fd, qs, ret = 0;
 	cluster_member_list_t *membership;
 	rg_state_list_t *rgs = NULL;
-	uint64_t local_node_id;
+	int local_node_id;
 	int fast = 0;
 	int runtype = 0;
+	cman_handle_t ch = NULL;
 
 	int refresh_sec = 0, errors = 0;
 	int opt, xml = 0, flags = 0;
 	char *member_name = NULL;
-	char *rg_name = NULL;
+	char *rg_name = NULL, real_rg_name[64];
 
 	while ((opt = getopt(argc, argv, "fIls:m:i:xvQh?")) != EOF) {
 		switch(opt) {
@@ -722,6 +897,12 @@ main(int argc, char **argv)
 
 		case 's':
 			rg_name = optarg;
+			if (!strchr(rg_name,':')) {
+				snprintf(real_rg_name,
+					 sizeof(real_rg_name),
+					 "service:%s", rg_name);
+				rg_name = real_rg_name;
+			}
 			break;
 
 		case 'x':
@@ -753,29 +934,32 @@ main(int argc, char **argv)
 		return 1;
 	}
 
+	signal(SIGPIPE, SIG_IGN);
+
 	/* Connect & grab all our info */
-	fd = clu_connect(RG_SERVICE_GROUP, 0);
+	ch = cman_init(NULL);
+	if (!ch) {
+		printf("CMAN is not running.\n");
+		return 1;
+	}
 
 	switch(runtype) {
 	case QSTAT_ONLY:
 		if (fd < 0)
 		       break;
-		ret = !(clu_quorum_status(RG_SERVICE_GROUP) &
-			QF_QUORATE);
+		ret = !(cman_is_quorate(ch));
 		goto cleanup;
 	case VERSION_ONLY:
 		printf("%s version %s\n", basename(argv[0]),
 		       PACKAGE_VERSION);
 		if (fd < 0)
 		       break;
-		printf("Connected via: %s\n", clu_plugin_version());
 		goto cleanup;
 	case NODEID_ONLY:
 		if (fd < 0)
 		       break;
-		clu_local_nodeid(NULL, &local_node_id);
-		printf("0x%08x%08x\n",(uint32_t)(local_node_id>>32),
-		       (uint32_t)(local_node_id&0xffffffff)); 
+		local_node_id = get_my_nodeid(ch);
+		printf("0x%08x\n",(uint32_t)(local_node_id)); 
 		goto cleanup;
 	}
 
@@ -789,11 +973,14 @@ main(int argc, char **argv)
 	signal(SIGTERM, term_handler);
 
 	while (1) {
-		qs = clu_quorum_status(RG_SERVICE_GROUP);
-		membership = build_member_list(&local_node_id);
+		qs = cman_is_quorate(ch);
+		membership = build_member_list(ch, &local_node_id);
 		
 		if (!member_name)
 			rgs = rg_state_list(local_node_id, fast);
+		if (rgs) {
+			flag_rgmanager_nodes(membership);
+		}
 
 		if (refresh_sec) {
 			setupterm((char *) 0, STDOUT_FILENO, (int *) 0);
@@ -810,7 +997,7 @@ main(int argc, char **argv)
 						 flags);
 
 		if (membership)
-			cml_free(membership);
+			free_member_list(membership);
 		if (rgs)
 			free(rgs);
 
@@ -821,6 +1008,6 @@ main(int argc, char **argv)
 	}
 
 cleanup:
-	clu_disconnect(fd);
+	cman_finish(ch);
 	return ret;
 }
